################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperAdaptiveCAI.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse

"""Wrapper for adaptive codon bias program
"""

import Experiment as E

class CAIError(Exception):
    pass

class AdaptiveCAIResult:

    def __init__(self):

        self.mIterations = 0
        self.mCodonUsageSphereStart = []        
        self.mCodonUsageSpheres = []
        self.mSetSizes = []
        self.mNumSequences = 0
        self.mWeights = []
        self.mFinalWeights = {}
        self.mGeneInfo = {}
        self.mCodonUsages = []

    def GetDominantThreshold( self, fraction = 0.1 ):
        """get CAI threshold for dominant set."""

        cai_values = map( lambda x: x["CAICLASS"], self.mGeneInfo.values())

        cai_values.sort()
        cai_values.reverse()

        return cai_values[int(len(cai_values) * fraction)]

    def Read( self, trace_file = None, gene_file = None, codon_file = None):
        
        """parse data from CAI output files."""
        
        if trace_file:
            iteration = None
            final_weights = None
            
            for line in trace_file:
                if line[:3] == "Got": continue

                if re.match( "CodonUsageIt", line ):
                    data = re.split("\s", line.strip())[2:]
                    self.mCodonUsageSpheres.append( map(float, data) )
                elif re.match( "CodonUsage sphere", line ):
                    data = re.split("\s", line.strip())[2:]
                    self.mCodonUsageSphereStart = map(float, data)
                elif re.match("First \d+ genes", line):
                    size = int(line.split(" ")[1])
                    self.mSetSizes.append( size )
                elif re.match("Number of points = \d+", line):
                    self.mNumSequences = int(line.split(" ")[4])
                elif re.match("Iteration \d", line):
                    iteration = int(line.split(" ")[1])
                    self.mWeights.append( {} )
                elif re.match("AdditionalInfoFile", line):
                    continue
                elif re.match("Looking", line):
                    continue
                elif re.match("Loaded", line):
                    continue
                elif re.match("Loading", line):
                   continue
                elif re.match("Gene", line):
                    continue
                elif re.match("External W-table", line):
                    ## ignore this table.
                    break
                elif re.match("GTFFinal", line):
                    final_weights = True
                    continue
                else:
                    data = re.split("\s", line.strip())
                    if iteration == None: continue
                    if final_weights:
                        for d in data:
                            codon, f = d.split(":")
                            self.mFinalWeights[codon.upper()] = float(f)
                    else:
                        for d in data:
                            codon, f = d.split(":")
                            self.mWeights[iteration][codon.upper()] = float(f)

        if gene_file:
            lines = gene_file.readlines()
            headers = []
            num_columns, num_rows = map(int, lines[0][:-1].split(" "))
            
            for x in range( 1, num_columns+1):
                header, type = lines[x][:-1].split( " ")
                header = header.upper()
                if header == "GENENAME":
                    genename_index = len(headers)
                headers.append((header, type))

            if num_columns+1+num_rows != len(lines):
                raise ValueError, "parsing error: not enough lines in %s" % gene_file

            for x in range( num_columns+1,num_columns+num_rows+1):
                data = re.split( "\s+", lines[x].strip())
                v = {}
                
                if len(data) != len(headers):
                    raise ValueError, "not enough fields in line %s" % lines[x]
                
                for c in range( len(headers) ):
                    if c == genename_index:
                        genename = data[c].replace( '"', '')
                        continue
                    
                    header,type = headers[c]

                    if type == "FLOAT":
                        v[header] = float(data[c])
                    elif type == "STRING":
                        v[header] = data[c]

                self.mGeneInfo[genename] = v

        if codon_file:

            for line in codon_file:

                data = re.split("\s+", line.strip())

                aa, codon = data[:2]

                if not self.mCodonUsages:
                    for x in range(len(data)-2):
                        self.mCodonUsages.append({})

                iteration = 0
                for v in data[2:]:

                    self.mCodonUsages[iteration][codon.upper()] = float(v)
                    iteration += 1

                
                
            
class AdaptiveCAI:

    mOptions = ""
    mIterations = 10
    mExecutable = "caijava"
    mTrace  = sys.stdout
    mStderr = sys.stderr
    mExtra  = sys.stdout
    mCodons = sys.stdout
    
    def __init__( self, iterations=10, options=""):
        
        self.mOptions = options
        self.mIterations = iterations

    def CreateTemporaryFiles( self ):
        """create temporary files."""
        self.mTempDirectory = tempfile.mkdtemp()
        
        self.mFilenameTempExtra = self.mTempDirectory + "/extra"
        self.mFilenameTempInput = self.mTempDirectory + "/input"

        ## note: this file is created automatically by caijava
        self.mFilenameTempCodons = self.mTempDirectory + "/codonusage"

        self.mFilenameTempWeights = self.mTempDirectory + "/weights"        

    def DeleteTemporaryFiles( self ):
        """clean up."""
        os.remove( self.mFilenameTempExtra )
        os.remove( self.mFilenameTempInput )
        os.remove( self.mFilenameTempCodons )
        if os.path.exists( self.mFilenameTempWeights ):
            os.remove( self.mFilenameTempWeights )                
        os.rmdir( self.mTempDirectory )

    def SetCodons( self, file = None):
        """set file for the information file."""
        self.mCodons = file
        
    def SetExtra( self, file = None):
        """set file for the information file."""
        self.mExtra = file
        
    def SetTrace( self, file = None):
        """set file for dumping stdout.

        If file is None, stdout is parsed but not dumped.
        """
        self.mTrace = file

    def SetStderr( self, file = None):
        """set file for dumping stderr."""
        self.mStderr= file
        
    def WriteOutput( self, lines, filename_output = None):
        """write output to file.

        If file is not given, lines are written to stdout.
        """
        
        if filename_output:
            outfile = open(filename_output, "w")
        else:
            outfile = sys.stdout
            
        outfile.write( string.join( lines, "") )

        if filename_output:
            outfile.close()

    def ParseResult( self, trace_file = None, information_file = None ):

        result = AdaptiveCAIResult()

        result.Read( trace_file, information_file )
        
        return result
    
    def RunOnFile( self, infile, infile_weights = None ):

        self.CreateTemporaryFiles()

        f = open(self.mFilenameTempInput, "w")
        lines = filter(lambda x: x[0] != "#", infile.readlines() )
        for line in lines:
            f.write(line)
        
        f.close()

        statement = string.join( ( self.mExecutable,
                                   self.mFilenameTempInput,
                                   "-i %i" % self.mIterations,
                                   "-s -g",
                                   "-f %s" % self.mFilenameTempExtra),
                                 " ")

        if infile_weights:
            f = open(self.mFilenameTempWeights, "w")
            for line in infile_weights: f.write(line)
            f.close()
            statement += " -ew %s" % self.mFilenameTempWeights

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempDirectory,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise CAIError, "Error in calculating CAI\n%s" % err
            
        self.mTrace.write( out )
        self.mStderr.write( err )
        self.mExtra.write( string.join(open(self.mFilenameTempExtra, "r").readlines(), ""))
        self.mCodons.write( string.join(open(self.mFilenameTempCodons, "r").readlines(), ""))

        self.DeleteTemporaryFiles()

    
    
     
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: WrapperAdaptiveCAI.py 756 2006-09-20 16:38:02Z andreas $")

    parser.add_option("-f", "--input-file", dest="input_filename", type="string",
                      help="input filename. If not given, stdin is used.",
                      metavar="FILE" )

    parser.add_option("-w", "--input-file-weights", dest="input_filename_weights", type="string",
                      help="input filename with weights. If not given, no CAIEXT values will be there.",
                      metavar="FILE" )

    parser.add_option ("-i", "--iterations", dest="iterations", type="int",
                       help="number of iterations." )

    parser.add_option("-o", "--output-file-trace", dest="output_filename_trace", type="string",
                      help="output filename for cai. If not given, no file is produced.",
                      metavar="FILE" )

    parser.add_option("-e", "--output-file-extra", dest="output_filename_extra", type="string",
                      help="output filename for extra information from cai. If not given, no file is produced.",
                      metavar="FILE" )

    parser.add_option("-c", "--output-file-codons", dest="output_filename_codons", type="string",
                      help="output filename for codon usage information.",
                      metavar="FILE" )


    parser.set_defaults( \
        input_filename = "-",
        input_filename_weights = None,
        output_filename_trace = None,
        output_filename_extra = None,
        output_filename_codons = None,                
        iterations = 10),

    (options, args) = E.Start( parser ) 

    wrapper = AdaptiveCAI( iterations=options.iterations )

    if options.input_filename == "-":
        file_stdin = sys.stdin
    else:
        file_stdin = open( options.input_filename, "r" )

    file_trace, file_extra, file_codons, file_weights = None, None, None, None
    
    if options.output_filename_trace:
        if options.output_filename_trace == "-":
            file_trace = sys.stdout
        else:
            file_trace = open( options.output_filename_trace, "w")
        wrapper.SetTrace( file_trace )
        
    if options.output_filename_extra:
        if options.output_filename_extra == "-":
            file_extra = sys.stdout
        else:
            file_extra = open( options.output_filename_extra, "w")
        wrapper.SetExtra( file_extra )

    if options.output_filename_codons:
        if options.output_filename_codons == "-":
            file_codons = sys.stdout
        else:
            file_codons = open( options.output_filename_codons, "w")
        wrapper.SetCodons( file_codons )

    if options.input_filename_weights:
        if options.input_filename_weights == "-":
            file_weights = sys.stdin
        else:
            file_weights = open( options.input_filename_weights, "r")

    wrapper.RunOnFile( file_stdin, infile_weights = file_weights)

    if file_codons and file_codons != sys.stdout:
        file_codons.close()
            
    if file_extra and file_extra != sys.stdout:
        file_extra.close()
        
    if file_trace and file_trace != sys.stdout:
        file_trace.close()

    if file_stdin and file_stdin != sys.stdin:
        file_stdin.close()

    if file_weights and file_weights != sys.stdin:
        file_weights.close()

    E.Stop()
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
mysql_copy_tables.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python mysql_copy_tables.py --help

Type::

   python mysql_copy_tables.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import random
import optparse
import CGAT.Experiment as E
import MySQLdb
import _mysql

# name, database 
data = (
    ( "Dmelanogaster", "dmel_vs_dmel4" ),
    ( "Dyakuba", "dyak_vs_dmel7" ),
    ( "Dpseudoobscura", "dpse_vs_dmel8" ),
    ( "Dvirilis", "dvir_vs_dmel6" ),
    ( "Dmojavanesis", "dmoj_vs_dmel6" ),
    ( "Dananassae", "dana_vs_dmel6" ),
    ( "Dgrimshawi", "dgri_vs_dmel5" ),
    ( "Dsimulans", "dsim_vs_dmel6" ),
    ( "Derecta", "dere_vs_dmel6" ),
    ( "Dsechellia", "dsec_vs_dmel3" ),
    ( "Dpersimilis", "dper_vs_dmel3" ),
    ( "Dwillistoni", "dwil_vs_dmel1" ),    
    )

#data = ( ( "Monodelphis", "mono2" ), )
#data = ( ( "Monodelphis", "mono_p1" ), )
#data = ( ( "Dsimulans", "dsim_vs_dmel6"), )

USAGE="""python gpipe/gbrowser_clone_devel.py

Clone the devel installation into the target installation.

The following cloing operations will be done:

%s
""" % (str(data))


class X:
    def __init__(self):
        pass

    def execute(self, msg):
        print msg

    def fetchall(self):
        return ( ("test",), )
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: mysql_copy_tables.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-t", "--target-dir", dest="target", type="string",
                      help="target directory.", metavar = "FILE"  )

    parser.set_defaults (
        connection = "db",
        source= "/var/www/conf/gbrowse-devel.conf",
        target= "/var/www/conf/gbrowse.conf",
        source_database = "gbrowser_devel_%s",
        target_database = "gbrowser_%s",        
    )

    (options, args) = E.Start( parser, add_mysql_options = True )

    dbhandle = MySQLdb.Connect( host = options.host,
                                user = options.user,
                                passwd = options.password,
                                port = options.port )
                                               
    cc = dbhandle.cursor()
    cc.execute("SHOW databases")
    databases = map(lambda x: x[0], cc.fetchall())
    cc.close()
    
    databases = map(lambda x: x[0], cc.fetchall())

    for name, schema in data:

        old_name = options.source_database % schema
        new_name = options.target_database % schema

        if old_name in databases:
            print "cloning: %s -> %s" % (old_name, new_name)

        cc = dbhandle.cursor()
        cc.execute("SHOW tables FROM %s" % old_name)
        tables = map(lambda x: x[0], cc.fetchall())        
        cc.close()

        ## I want to do this without file system access and
        ## without delay
        ## 1. delete target database
        ## 2. recreate target database
        ## 3. rename tables to target database
        ## 4. recreate tables in source database
        ## 5. copy data to source database
        cc = dbhandle.cursor()
        if new_name in databases:
            cc.execute( "DROP DATABASE %s" % new_name )

        cc.execute( "CREATE DATABASE %s" % new_name )
        
        cc = dbhandle.cursor()
        for table in tables:
            cc.execute( "ALTER TABLE %s.%s RENAME AS %s.%s" % ( old_name, table, new_name, table))

        print "copying has finished."
        cc.close()

        cc = dbhandle.cursor()
        cc.execute( "USE %s" % old_name )
        
        for table in tables:
            print "recreating table %s in %s" % (table, old_name)
            cc.execute( "SHOW CREATE TABLE %s.%s" % (new_name, table) )
            create_statement = cc.fetchone()[1]
            cc.execute( create_statement )
            cc.execute( "INSERT INTO %s.%s SELECT * FROM %s.%s" % (old_name, table, new_name, table ))
            
        cc.close()
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
codeml2tsv.py - analyze results from codeml kaks run
=============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------


Usage
-----

Example::

   python codeml2tsv.py --help

Type::

   python codeml2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse

from types import *

import CGAT.Genomics as Genomics
import CGAT.Experiment as E
import CGAT.WrapperCodeML as WrapperCodeML
import scipy
import scipy.stats
import CGAT.TreeTools as TreeTools

import CGAT.Stats as Stats

if __name__ == "__main__":
    
    parser = E.OptionParser( version = "%prog version: $Id: codeml2tsv.py 2781 2009-09-10 11:33:14Z andreas $" )

    parser.add_option("-m", "--methods", dest="methods", type="string",
                      help="""methods for analysis.
write-ks-tree: write out ks tree(s).
write-ka-tree: write out ka tree(s).
                      """  )

    parser.add_option("--prefix", dest="prefix", type="string",
                      help="prefix for rows.")

    parser.add_option("--pattern-input-filenames", dest="pattern_input_filenames", type="string",
                      help="input pattern."  )

    parser.add_option("--filter-probability", dest="filter_probability", type="float",
                      help="threshold for probability above which to include positive sites."  )

    parser.add_option("--filter-omega", dest="filter_omega", type="float",
                      help="threshold for omega above which to include positive sites."  )

    parser.add_option("--models", dest="models", type="string",
                      help="restrict output to set of site specific models."  )

    parser.add_option("--significance-threshold", dest="significance_threshold", type="float",
                      help="significance threshold for log-likelihood test."  )

    parser.add_option("--mode", dest="mode", type="choice",
                      choices=("pairs", "1xn" ),
                      help="analysis mode."  )

    parser.set_defaults(
        methods = "",
        prefix = None,
        filter_probability = 0,
        filter_omega = 0,
        models = "",
        significance_threshold = 0.05,
        mode = "pairs",
        )

    (options, args) = E.Start( parser )

    options.methods = options.methods.split(",")
    options.models = options.models.split(",")

    codeml = WrapperCodeML.CodeML()

    results = []
    if len(args) == 0:
        # read from stdin, if no arguments are given
        results.append( codeml.parseOutput( sys.stdin.readlines() ))
    else:
        # read multiple results
        for f in args:
            try:
                results.append(codeml.parseOutput( open(f, "r").readlines() ))            
            except WrapperCodeML.ParsingError, msg:
                options.stdlog.write("# parsing error in file %s: %s.\n" % (f, msg))
                continue
            
    if options.prefix:
        prefix_tree = ">%s\n" % options.prefix
        prefix_header = "prefix\t"
        prefix_row = "%s\t" % options.prefix
    else:
        prefix_tree = ""
        prefix_header = ""
        prefix_row = ""
        
    for method in options.methods:

        if method == "write-ks-tree":
            for result in results:
                options.stdout.write(prefix_tree + TreeTools.Tree2Newick(result.mTreeKs) + "\n" )
                
        elif method == "write-ka-tree":
            for result in results:
                options.stdout.write(prefix_tree + TreeTools.Tree2Newick(result.mTreeKa) + "\n" )

        elif method == "write-kaks-tree":
            for result in results:
                options.stdout.write(prefix_tree + TreeTools.Tree2Newick(result.mTreeKaks) + "\n" )

        elif method == "lrt":
            ## perform log-likelihood ratio test between successive models
            ## Assumption is that the models are nested with the previous model
            ## being the less complex model.
            first_result = results[0]
            last_result = results[0]
            x = 1

            options.stdout.write("%sm1\tm2\tstatus\tlnL1\tnp1\tlnl2\tnp2\tP-value\n" % prefix_header )
            
            for result in results[1:]:

                if options.mode == "pairs":
                    reference_result = last_result
                    reference_id = x - 1
                elif options.mode == "1xn":
                    reference_result = first_result
                    reference_id = 0
                    
                if reference_result.mNumParameters >= result.mNumParameters:
                    if options.loglevel >= 1:
                        options.stdlog.write ( "number of parameters of full model not increased (null=%i, full=%i).\n" % (reference_result.mNumParameters, result.mNumParameters))
                    continue

                lrt = Stats.doLogLikelihoodTest(
                    result.mLogLikelihood, result.mNumParameters,                     
                    reference_result.mLogLikelihood, reference_result.mNumParameters,
                    options.significance_threshold )
                
                if lrt.mPassed:
                    c = "passed"
                else:
                    c = "failed"

                options.stdout.write("%s%i\t%i\t%s\t%f\t%i\t%f\t%i\t%5.2e\n" % (prefix_row, reference_id, x, c,
                                                                                lrt.mFullLogLikelihood, lrt.mFullNumParameters,
                                                                                lrt.mNullLogLikelihood, lrt.mNullNumParameters,
                                                                                lrt.mProbability,
                                                                                ))
                
                last_result = result
                x += 1
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
substitute_tokens.py - substitute fields in tables
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script substitutes the elements within columns within an table.


Usage
-----

Example::

   python substitute_tokens.py --help

Type::

   python substitute_tokens.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse

import CGAT.IOTools as IOTools
import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv


    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-c", "--create", dest="create", type="string",
                      help="create substitution list [%default] " )

    parser.add_option("-r", "--regex-token", dest="regex_token", type="string",
                      help="regular expression for tokens (has to create one pair of brackets) [%default] " )

    parser.add_option("-p", "--pattern-sub", dest="pattern_sub", type="string",
                      help="pattern for substitution [%default] " )

    parser.add_option("-a", "--apply", dest="apply", type="string",
                      help="apply substitution list [%default] " )

    parser.add_option("-x", "--extended", dest="extended", action="store_true",
                      help="replace not just with second column in map, but all columns. [%default] " )

    parser.add_option("-i", "--invert", dest="invert", action="store_true",
                      help="pairs of substitution patterns is to,from [%default] " )

    parser.add_option("-m", "--multiple", dest="multiple", action="store_true",
                      help="do multiple substitutions per row [%default] " )

    parser.add_option("-e", "--echo", dest="echo", action="store_true",
                      help="echo susbstituted column [%default] " )

    parser.add_option("-k", "--keep", dest="keep", action="store_true",
                      help="keep column that is substituted [%default] " )

    parser.add_option("-f", "--filter", dest="filter", action="store_true",
                      help="remove lines not matching [%default] " )

    parser.add_option("-y", "--reverse-filter", dest="reverse_filter", action="store_true",
                      help="remove lines matching [%default] " )

    parser.add_option("-n", "--inplace", dest="inplace", action="store_true",
                      help="do inplace subsitutions of all files on command line [%default] " )

    parser.add_option("-b", "--backup", dest="backup", action="store_true",
                      help="keep backup (with ending .bak) [%default] " )

    parser.add_option("--keep-header", dest="keep_header", action="store_true",
                      help="do not apply transformation to header [%default] " )

    parser.add_option("-o", "--columns-token", dest="columns_token", type="string",
                      help="substitute tokens in columns [%default] " )

    parser.add_option("-s", "--select-rows", dest="regex_rows", type="string",
                      help="regular expression for rows to use. [%default] " )

    parser.set_defaults( create = None,
                         regex_token = None,
                         pattern_sub = "%s",
                         apply = None,
                         invert = False,
                         multiple = False,
                         columns_token = None,
                         filter = None,
                         reverse_filter = None,
                         inplace = False,
                         backup = False,
                         regex_rows = None,
                         echo = False,
                         extended = False,
                         keep = False,
                         keep_header = False )

        

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )


    if options.regex_token:
        options.regex_token = re.compile( options.regex_token )

    if options.regex_rows:
        options.regex_rows = re.compile( options.regex_rows )

    if options.columns_token:
        if options.columns_token != "all":
            options.columns_token = map(lambda x: int(x) - 1,string.split(options.columns_token, ","))

    file_id = 0

    keys = {}

    if options.apply:
        infile = IOTools.openFile(options.apply, "r")
        for line in infile:
            if line[0] == "#": continue

            d = line[:-1].split("\t")
            try:
                a, b = d[:2]
            except ValueError:
                print "# invalid map skipped in line: %s" % line
                continue
            
            if options.invert:            
                a, b = b, a
                if options.extended:
                    b = "\t".join(d[0] + d[2:])
            else:
                if options.extended:
                    b = "\t".join(d[1:])
                    
            if not keys.has_key( a ):
                keys[a] = []

            if options.keep:
                b = a + "\t" + b

            keys[a].append( b )
            
    files = args
    
    if not options.inplace and len(args) == 0:
        files = ["-"]
        
    for file in files:

        close_infile = False
        close_outfile =False
        if file == "-":
            infile = sys.stdin
            outfile = sys.stdout
        else:
            if options.inplace:
                os.rename( file, file + ".bak" )
                infile = IOTools.openFile( file + ".bak", "r")
                outfile = IOTools.openFile( file, "w")
                close_infile = True
                close_outfile = True
            else:
                infile = IOTools.openFile( file, "r")
                outfile = sys.stdout
                close_infile = True

        first = True
        
        for line in infile:
            if line[0] == "#":
                outfile.write(line)
                continue

            if first:
                first = False
                if options.keep_header:
                    outfile.write(line)
                    continue

            if options.regex_rows:
                if options.regex_rows.search( line ):
                    outfile.write(line)
                    continue
                
            new_lines = []
            if options.regex_token:
                r = options.regex_token.search( line[:-1] )
                while r:
                    key = r.group(1)
                    if key not in keys:
                        if options.create:
                            keys[key] = [options.pattern_sub % str(len(keys))]
                        else:
                            new_lines.append(line[:-1])
                            break

                    for k in keys[key]:
                        new_lines.append( line[:r.start(1)] + k + line[r.end(1):-1] )

                    if options.multiple:
                        r = options.regex_token.search( line[r.end(1):-1] )
                    else:
                        break
                else:
                    if not options.filter:
                        new_lines.append(line[:-1])

            elif options.columns_token:
                data = line[:-1].split("\t")
                if options.columns_token == "all":
                    columns = range( len(data))
                else:
                    columns = options.columns_token
                keep = not options.reverse_filter
                first_multiple = True
                for c in columns:
                    k = data[c]
                    if k in keys:
                        if len(keys[k]) > 1:
                            if not first_multiple: 
                                raise "warning: could not substitute multiple keys for %s in multiple columns in line: %s" % (k, line)
                            first_multiple = False
                        for v in keys[k]:
                            if options.echo: data.append(data[c])
                            ## multiple substitutions: write data now
                            data[c] = v
                            if keep:
                                new_lines.append( string.join( data, "\t" ) )
                        keep = False
                    else:
                        if options.create:
                            keys[k] = [options.pattern_sub % str(len(keys))]
                            data[c] = keys[k][0]
                        elif options.filter:
                            keep = False
                        elif options.reverse_filter:
                            keep = True
                if keep:
                    new_lines.append(string.join(data, "\t"))

            elif options.apply:
                for key in keys:
                    for k in keys[key]:
                        line = line.replace( key, k )
                new_lines.append( line[:-1] )

            if new_lines:
                outfile.write(string.join(new_lines, "\n") + "\n")

        if options.create:
            create_file = IOTools.openFile(options.create, "w")
            for key in keys:
                for k in keys[key]:
                    create_file.write("%s\t%s\n" % (key, str(k)))
            create_file.close()

        if close_outfile: outfile.close()
        if close_infile: infile.close()
    
        if options.inplace and not options.backup:
            os.remove( file + ".bak" )

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph2stats.py - calculate statistics for a (redundant) graph 
=============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a :term:`graph` in :term:`edge list` format and 
computes stats on each edges. This only make sense, if the graph
contains multiple edges between a pair of vertices.


Usage
-----

Example::

   python graph2stats.py --help

Type::

   python graph2stats.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

""" program $Id: graph2stats.py 2782 2009-09-10 11:40:29Z andreas $

python graph2stats.py < graph.in

calculate statistics for a (redundant) graph.

Possibilities are: 
"""
import CGAT.Experiment as E
import CGAT.Histogram as Histogram

def PrintHistograms( outfile, titles, histograms, options ):

    combined_histogram = Histogram.Combine( hists )

    outfile.write( "\t".join( ("bin",) + titles ) )
    Histogram.Print( combined_histogram, nonull = options.nonull )        

    
    

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: graph2stats.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-r", "--range", dest="range", type="string",
                      help="range to calculate histogram for."  )
    parser.add_option("-b", "--bin-size", dest="bin_size", type="string",
                      help="bin size."  )
    parser.add_option("-i", "--titles", dest ="titles", action="store_true",
                      help="use supplied column titles." )
    parser.add_option("-s", "--make-symmetric", dest ="make_symmetric", action="store_true",
                      help="symmetrize graph." )
    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take for calculating histograms." )
    parser.add_option("-p", "--output-pattern", dest="output_pattern", type="string",
                      help="pattern for output files." )
    parser.add_option("-m", "--method", dest="method", type="string",
                      help="method." )
    parser.add_option("-o", "--output-format", dest="output_format", type="string",
                      help="output format." )
    
    parser.add_option("--min-value", dest="min_value", type="float",
                      help="minimum value for histogram.")
    parser.add_option("--max-value", dest="max_value", type="float",
                      help="maximum value for histogram.")
    
    parser.set_defaults(
        bin_size = None,
        range = None,
        titles = False,
        columns = "all",
        append = (),
        empty_bins = False,
        min_value = None,
        max_value = None,
        normalize = False,
        cumulative = False,
        reverse_cumulative = False,
        nonull = None,
        make_symmetric = False,
        output_pattern = "%s.hist",
        method = "histograms",
        output_format = "semi"
        )

    (options, args) = E.Start( parser )

    if options.columns != "all":
        options.columns = map(lambda x: int(x) -1 , options.columns.split(","))

    if options.range:
        options.min_value, options.max_value = map(float, options.range(split(",")))
        
    # retrieve data
    lines = filter( lambda x: x[0] <> "#", sys.stdin.readlines())

    vals = {}

    if options.method == "histograms":

        ## read data
        for line in lines:

            v1, v2, w = line[:-1].split("\t")[:3]

            try:
                w[3] = float(w[3])
            except ValueError:
                nerrors += 1
                continnue

            if v1 not in vals: vals[v1] = {}
            if v2 not in vals[v1]: vals[v1][v2] = []
            vals[v1][v2].append( w )
            if options.make_symmetric:
                if v2 not in vals: vals[v2] = {}
                if v1 not in vals[v2]: vals[v2][v1] = []
                vals[v2][v1].append( w )

        ## convert to histograms
        for k1, vv in vals.items():
            for k2 in vv.keys():
                if len(vv[k2]) == 0: continue

                h = Histogram.Calculate( vv[k2],
                                         no_empty_bins = options.empty_bins,
                                         increment = options.bin_size,
                                         min_value = options.min_value,
                                         max_value = options.max_value)
                
                if options.normalize: h = Histogram.Normalize( h )
                if options.cumulative: h = Histogram.Cumulate( h )
                if options.reverse_cumulative: h = Histogram.Cumulate( h, direction = 0 )

                vv[k2] = h
        
        ## write output
        if options.output == "semi":
            for k1, vv in vals.items():

                outfile = open( options.output_pattern % k1 )

                kk2 = vv.keys()
                kk2.sort()

                hists = []
                for k2 in kk2: hists.append( vv[k2] )

                PrintHistograms( outfile, kk2, hists, options )

                outfile.close()
                
    elif options.method == "counts":

        ## read data
        for line in lines:

            v1, v2 = line[:-1].split("\t")[:2]

            if v1 not in vals: vals[v1] = {}
            if v2 not in vals[v1]: vals[v1][v2] = 0
            vals[v1][v2] += 1
            if options.make_symmetric:
                if v2 not in vals: vals[v2] = {}
                if v1 not in vals[v2]: vals[v2][v1] = 0
                vals[v2][v1] += 1

        ## convert to histograms
        for k1, vv in vals.items():
            for k2 in vv.keys():
                options.stdout.write( "%s\t%s\t%i\n" % (k1, k2, vv[k2] ) )
                
    E.Stop()




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperDialign.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess

"""Wrapper for Dialign.
"""

import Genomics

class Dialign:

    mOptions = ""
    mExecutable = "dialign -stdo "
    mEnvironment = "DIALIGN2_DIR=/net/cpp-group/legacy/bin/dialign2_dir; export DIALIGN2_DIR;"
    
    def __init__( self, options ):
        self.mOptions = options

    def Align( self, s1, s2, result ):

        result.clear()
        
        handle_tmpfile, filename_tmpfile = tempfile.mkstemp()
        os.write( handle_tmpfile, ">s1\n%s\n" % (s1))
        os.write( handle_tmpfile, ">s2\n%s\n" % (s2))
        os.close( handle_tmpfile )

        statement = string.join( ( "(", self.mEnvironment, 
                                   self.mExecutable, self.mOptions, filename_tmpfile, ")" ),
                                 " ")

        p = subprocess.Popen( statement , 
                              shell=True, 
                              stdin=subprocess.PIPE, 
                              stdout=subprocess.PIPE, 
                              stderr=subprocess.PIPE, 
                              close_fds=True)

        (file_stdout, file_stdin, file_stderr) = (p.stdin, p.stdout, p.stderr)
        file_stdin.close()
        lines = file_stdout.readlines()
        lines_stderr = file_stderr.readlines()
        exit_code = file_stdout.close()
        file_stderr.close()
        
        if exit_code:
            raise "Error while executing statement %s" % statement

        r = None
        for x in range(len(lines)):
            if re.search( "Alignment \(FASTA format\):", lines[x]):
                r = Genomics.ParseFasta2Hash( lines[x+2:])
                break

        if not r: return None
        
        a1 = r['s1']
        a2 = r['s2']

        x1 = 1
        x2 = 1
        for pos in range(len(a1)):
            if a1[pos] in string.uppercase and a2[pos] in string.uppercase:
                result.addPairExplicit( x1, x2, 0 )
                x1 += 1
                x2 += 1
                continue
            
            if a1[pos] != "-": x1 += 1
            if a2[pos] != "-": x2 += 1            


        os.remove( filename_tmpfile )
        
        return result
                          
            
        
                                 
        
        
        
        
    
        



################################################################################
#   Gene prediction pipeline 
#
#   $Id: Tree.py 2784 2009-09-10 11:41:14Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
Tree.py - A derivation of the tree class from Biopython
=======================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

"""
import sys, string, re, os, math

import Bio.Nexus, Bio.Nexus.Nodes, Bio.Nexus.Trees
import TreeTools

def updateNexus( nexus ):
    """change trees in nexus object to trees from here.

    This a quick patch until I have written a better routine to read the tree
    directly from a file.
    """
    for x in range(len(nexus.trees)):
        ## remove the trailing ";"
        t = nexus.trees[x]

        ## in Biopython, to_string includes the name separated by =, while
        ## my own class does not.
        tree = t.to_string( branchlengths_only = False, plain=False)
        a = tree.find("=")
        if a >= 0: tree = tree[a+1:]

        nexus.trees[x] = Tree( tree=tree.strip(),
                               weight=t.weight,
                               rooted=t.rooted,
                               name=t.name,                                
                               data=t.dataclass,
                               max_support=t.max_support)

##-------------------------------------------------------------------------
## Empty function for tree traversal
def Nop( x ): return True

        
##-------------------------------------------------------------------------
class Tree( Bio.Nexus.Trees.Tree ):

    def __init__(self, *args, **kwargs ):
        Bio.Nexus.Trees.Tree.__init__( self, *args, **kwargs )

    def __len__(self):
        """returns the # of nodes in the tree."""
        return len(self.chain.keys())

    def root_at_node(self, node, distance = 0 ):
        """root tree at node. Distance gives distance of node to new root.

        This is a subset of the code taken from root_with_outgroup.
        """

        def _connect_subtree(parent,child):
            """Hook subtree starting with node child to parent."""
            for i,branch in enumerate(self.unrooted):
                if parent in branch[:2] and child in branch[:2]:
                    branch=self.unrooted.pop(i)
                    break 
            else:
                raise TreeError, 'Unable to connect nodes for rooting: nodes %d and %d are not connected' % (parent,child)
            self.link(parent,child)
            self.node(child).data.branchlength=branch[2]
            self.node(child).data.support=branch[3]
            #now check if there are more branches connected to the child, and if so, connect them
            child_branches=[b for b in self.unrooted if child in b[:2]]
            for b in child_branches:
                if child==b[0]:
                    succ=b[1]
                else:
                    succ=b[0]
                _connect_subtree(child,succ) 

        ## code for getting the outgroup node has been removed. Instead
        ## outgroup_node is set to node.
        outgroup_node = node

        if outgroup_node==self.root:
            return self.root
        
        # find ingroup node on a rooted tree
        ingroup_node = self.node(outgroup_node).prev

        # if tree is already rooted with outgroup on a bifurcating root,
        if ingroup_node == self.root and len(self.node(self.root).succ)==2:
            s = self.node(ingroup_node).succ
            if s[0] == outgroup_node:
                ingroup_node = s[1]
            else:
                ingroup_node = s[0]
                
        # move to unrooted structure
        self.unroot()

        # now we find the branch that connects outgroup and ingroup
        # print self.node(outgroup_node).prev
        for i,b in enumerate(self.unrooted):
            if outgroup_node in b[:2] and ingroup_node in b[:2]:
                root_branch=self.unrooted.pop(i)
                break
        else:
            raise TreeError, 'Unrooted and rooted Tree do not match'

        # now we destroy the old tree structure, but keep node data. Nodes will be reconnected according to new outgroup
        for n in self.all_ids():
            self.node(n).prev=None
            self.node(n).succ=[]

        # now we just add both subtrees (outgroup and ingroup) branch for branch
        root=Bio.Nexus.Nodes.Node(data=Bio.Nexus.Trees.NodeData())            # new root    
        self.add(root)                              # add to tree description
        self.root=root.id                           # set as root

        self.unrooted.append([root.id,ingroup_node,root_branch[2]-distance,root_branch[3]])  # add branch to ingroup to unrooted tree
        self.unrooted.append([root.id,outgroup_node,distance,0.0])   # add branch to outgroup to unrooted tree
        _connect_subtree(root.id,ingroup_node)      # add ingroup
        _connect_subtree(root.id,outgroup_node)     # add outgroup
        
        # if theres still a lonely node in self.chain, then it's the old root, and we delete it
        oldroot=[i for i in self.all_ids() if self.node(i).prev is None and i!=self.root]
        if len(oldroot)>1:
            raise TreeError, 'Isolated nodes in tree description: %s' % ','.join(oldroot)
        elif len(oldroot)==1:
            self.kill(oldroot[0])
        return self.root
        
    def to_string(self,support_as_branchlengths=False,
                  branchlengths_only=False,plain=True,
                  write_all_taxa=False,
                  branchlength_format= "%1.5f",
                  support_format = "%1.2f",
                  format = "nexus" ):
        """Return a paup compatible tree line.
       
        to_string(self,support_as_branchlengths=False,branchlengths_only=False,plain=True)

        Added by AH
           if write_all_taxa is set to true, internal node names are output as well
           NHX format for bootstrap values and species names.
        """
        # if there's a conflict in the arguments, we override plain=True
        if support_as_branchlengths or branchlengths_only:
            plain=False
        self.support_as_branchlengths=support_as_branchlengths
        self.branchlengths_only=branchlengths_only
        self.plain=plain
        self.write_all_taxa = write_all_taxa
        
        def make_info_string(data,terminal=False):
            """Creates nicely formatted support/branchlengths."""
            # CHECK FORMATTING

            if format == "nhx":

                info = "[&&NHX"
                try:
                    info += ":S=%s" % data.species
                except AttributeError:
                    pass
                
                if not terminal:
                    info += ":B=%s" % (support_format % data.support)
                return ':%s%s]' % (branchlength_format % data.branchlength, info)                        
            
            if self.plain: # plain tree only. That's easy.
                return ''
            elif self.support_as_branchlengths: # support as branchlengths (eg. PAUP), ignore actual branchlengths
                if terminal:    # terminal branches have 100% support
                    return ':%1.2f' % (support_format % self.max_support)
                else:
                    return ':%1.2f' % (support_format % data.support)
            elif self.branchlengths_only: # write only branchlengths, ignore support
                return ':%s' % (branchlength_format % data.branchlength)
            else:   # write suport and branchlengths (e.g. .con tree of mrbayes)
                if format in ("nh", "nexus"):
                    if terminal:
                        return ':%s' % (branchlength_format % data.branchlength)
                    else:
                        if data.branchlength is not None and data.support is not None:  # we have blen and suppport
                            sup = support_format % data.support
                            bl = branchlength_format % data.branchlength
                        elif data.branchlength is not None:                             # we have only blen
                            sup = support_format % 0
                            bl = branchlength_format % data.branchlength
                        elif data.support is not None:                                  # we have only support
                            sup = support_format % data.support
                            bl = branchlength_format % 0
                        else:
                            sup = support_format % 0
                            bl = branchlength_format % 0
                        return "%s:%s" % (sup, bl)
                            
        def newickize(node):
            """Convert a node tree to a newick tree recursively."""
            if not self.node(node).succ:    #terminal
                return self.node(node).data.taxon+make_info_string(self.node(node).data,terminal=True)
            else:
                return '(%s)%s' % (','.join(map(newickize,self.node(node).succ)),make_info_string(self.node(node).data))
            return subtree

        def newickize_all_taxa( node ):
            if not self.node(node).succ:    # terminal
                return self.node(node).data.taxon+make_info_string(self.node(node).data,terminal=True)
            else:
                if self.node(node).data.taxon != None:
                    ## changed output: first taxon name, then branch length
                    return '(%s)%s%s' % (','.join(map(newickize_all_taxa,self.node(node).succ)),self.node(node).data.taxon, make_info_string(self.node(node).data))
                else:
                    return '(%s)%s' % (','.join(map(newickize_all_taxa,self.node(node).succ)),make_info_string(self.node(node).data))
            return subtree

        treeline = ''
        if format == "nexus":
            treeline='tree '
            if self.name:
                treeline+=self.name
            else:
                treeline+='a_tree'

            treeline+=' = '
            if self.weight<>1:
                treeline+='[&W%s] ' % str(round(float(self.weight),3))
            if self.rooted:
                treeline+='[&R] '
            
        if self.write_all_taxa:
            treeline+='(%s);' % ','.join(map(newickize_all_taxa,self.node(self.root).succ))            
        else:
            treeline+='(%s);' % ','.join(map(newickize,self.node(self.root).succ))
        return treeline 

    ##-------------------------------------------------------------------------
    def get_nodes( self, node_id ):
        """Return a list of nodes downwards from a node (self, node_id).

        The list includes the given node_id.
        """
        if node_id is None:
            node_id=self.root
        if node_id not in self.chain:
            raise TreeError('Unknown node_id: %d.' % node_id)    
        list=[node_id]
        for succ in self.chain[node_id].succ:
            list.extend(self.get_nodes(succ))
        return list

    ##-------------------------------------------------------------------------
    def get_leaves( self, node_id ):
        """Return a list of leaf nodes downward from a node (self, node_id).
        """
        return filter( lambda x: self.node(x).succ == [], self.get_nodes(node_id))

    ##-------------------------------------------------------------------------
    def root_midpoint( self ):
        """perform midpoint rooting of tree.

        The root is placed at equal distance to the two leaves
        furthest apart in the tree (centroid of the tree).
        """

        nnodes = max( self.chain.keys() ) + 1

        ## distance to root of parent including
        ## the branch length of node to parent
        map_N2Root   = [0] * (nnodes)
        ## distance for node to leaves excluding branch length of this node
        map_N2Leaves = [0] * (nnodes)
        ## distance for node to other part of trees excluding its children
        map_N2Other = [0] * (nnodes)
        
        def dist2leaves( node_id ):
            node = self.node(node_id)
            if node.succ:
                map_N2Leaves[node_id] = max( [map_N2Leaves[s] + self.node(s).data.branchlength for s in node.succ ] )

        def dist2root( node_id ):
            node = self.node(node_id)
            map_N2Root[node_id] = node.data.branchlength        
            if node.prev != None:
                map_N2Root[node_id] += map_N2Root[node.prev] 

        ## traverse tree and record distance to root and maximum distance to leaf at each node
        self.dfs( self.root,
                  pre_function = dist2root,
                  post_function = dist2leaves )
        
        ## update distance to root for each node with length to leaves on the other part of the tree.
        s = set(self.node(self.root).succ)
        for x in s:
            d = max( [map_N2Leaves[i] + self.node(i).data.branchlength for i in s.difference( set((x,))) ] )
            for xx in self.get_nodes( x ):
                map_N2Root[xx] += d

        ## update distance to any other
        def dist2other( node_id ):
            node = self.node(node_id)
            if node.prev != None:
                p = self.node(node.prev)
                root_dist = map_N2Root[node.prev] 
                s = set(p.succ)
                d = max( [map_N2Leaves[i] + self.node(i).data.branchlength for i in s.difference( set((node_id,))) ] )
                max_d = max( (d, root_dist, map_N2Other[node.prev] + p.data.branchlength ) )
                map_N2Other[node_id] = max_d

        self.dfs( self.root,
                  pre_function = dist2other )
            
#         self.display()                
#         for x in range(0, nnodes):
#             node = self.node(x)            
#             balance = map_N2Other[x] - map_N2Leaves[x] 
#             print "x=",x, "bl=", node.data.branchlength, \
#                   "d2root=", map_N2Root[x], "d2leaves=", map_N2Leaves[x], "d2other=", map_N2Other[x], "balance=", balance
            
        # note: need not treat root. Set d2leaves > d2root
        # so that it is never chosen selected.
        map_N2Leaves[self.root] = 1
        map_N2Root[self.root] = 0

        ## find node with minimum balance
        best_node = None
        best_balance = None
        for x in range(0, nnodes):
            if x == self.root: continue
            node = self.node(x)
            balance = math.fabs(map_N2Other[x] - map_N2Leaves[x])
            # need to add a small quantity to node.data.branchlength 
            # to avoid rounding errors for large trees with uniform branchlengths
            if balance >= 0 and balance <= node.data.branchlength + 0.001:
                if best_balance == None or best_balance > balance:
                    best_balance = balance
                    best_node = x
        
        ## in trees with all 0 distances, best_node will be None
        ## in this case return a balanced tree
        if best_node == None:
            return self.root_balanced()

        ## compute distance of current node to new node
        d1 = map_N2Leaves[best_node]

        distance = float(map_N2Other[best_node] - map_N2Leaves[best_node] + self.node(best_node).data.branchlength) / 2.0 

        balance = map_N2Other[best_node] - map_N2Leaves[best_node]
        distance = float(balance + self.node(best_node).data.branchlength) / 2.0 

        ## remove current root
        self.unroot()

        ## reroot tree
        result = self.root_at_node( best_node, distance = distance )

    ##-------------------------------------------------------------------------
    def getNumLeaves( self ):
         """return list with number of leaves beyond each node
         """
         nnodes = max( self.chain.keys() ) + 1

         counts = [0] * nnodes
    
         def count( node_id ):
             s = self.node(node_id).succ
             if s == []:
                 counts[node_id] = 1
             else:
                 for n in s:
                     counts[node_id] += counts[n]

         self.dfs( self.root, post_function=count)
         return counts

    ##-------------------------------------------------------------------------
    def root_balanced( self ):
        """perform balanced rooting of tree.

        The root is placed such that the number of leaves
        on either side of the tree is equal (or at most
        different by 1).
        """
        nnodes = max( self.chain.keys() ) + 1
        nleaves = len(self.get_terminals())

        ## number of leaves
        map_num_leaves = self.getNumLeaves()

        ## simply find a node whose number of leaves is nleaves / 2
        threshold = int ( math.floor( nleaves / 2.0 ) )
        best_node = None
        for x in range(nnodes):
            if map_num_leaves[x] == threshold:
                best_node = x
                break
        
        if best_node == None:
            raise "no best node found for %i leaves" % threshold
        
        ## remove current root
        self.unroot()
        
        ## reroot tree
        result = self.root_at_node( best_node )

    ##-------------------------------------------------------------------------
    def dfs( self, node_id,
             pre_function = Nop,
             descend_condition = Nop,
             post_function = Nop):
        """dfs tree tree traversal starting at node_id.

        Apply functions pre_function at first and
        post_function at last visit of a node.
        """
        pre_function( node_id )
        for n in self.node(node_id).succ:
            if descend_condition( node_id ):
                self.dfs( n, pre_function, descend_condition,post_function )
        post_function( node_id )


    ##-------------------------------------------------------------------------
    def writeToFile( self, outfile, with_branchlengths = True, format = "nh" ):
        """write a tree to a file."""
        
        if format == "nh":
            outfile.write( ">%s\n%s\n" % (self.name,
                           self.to_string( branchlengths_only = with_branchlengths,
                                           write_all_taxa = True,
                                           format = "nh" ) ))
            
        elif format == "nhx":
            outfile.write( ">%s\n%s\n" % (self.name,
                           self.to_string( format = "nhx" ) ))
            
    ##-------------------------------------------------------------------------
    def truncate( self, node_id, taxon = None, keep_node = None ):
        """truncate tree at node_id.

        This function will not change any branch lengths.
        If keep is given, single child nodes will be collapsed
        until keep_node is reached.
        """
        succ = tuple(self.node(node_id).succ)
        if len(succ) == 0:
            ## if this is a leaf, do the same as tree.prune, but
            ## nod not change branch length
            prev=self.unlink(node_id)
            self.kill(node_id)
            if not prev==self.root and len(self.node(prev).succ)==1 \
                    and prev != keep_node:
                succ=self.node(prev).succ[0]
                self.collapse(prev)
        else:
            for s in succ:
                self.unlink(s)
                self.kill(s)

        if taxon: self.node(node_id).data.taxon = taxon

    ##-------------------------------------------------------------------------
    def relabel( self, map_old2new, warn = False ):
        """relabel taxa in tree using the provided mapping.
        """
        for node_id, node in self.chain.items():
            if not node.data.taxon: continue

            if node.data.taxon in map_old2new:
                node.data.taxon = map_old2new[node.data.taxon]
            else:
                if warn:
                    raise KeyError( "taxon %s not in map" % node.data.taxon )

    ##-------------------------------------------------------------------------
    def rescaleBranchLengths( self, value ):
        """rescale branch length so that they sum up to value."""

        t = 0.0
        for node_id, node in self.chain.items():
            t += node.data.branchlength
        if t == 0: return
        for node_id, node in self.chain.items():
            node.data.branchlength /= t
        return



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
medip_merge_intervals.py - merge differentially methylated regions
==================================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes the output of DESeq or EdgeR and merges
adjacent intervals that show a similar expression change.

Input is data like this::

   contig start end treatment_name  treatment_mean  treatment_std   control_name    control_mean    control_std     pvalue  qvalue  l2fold  fold    significant     status                                                                                 
   chr1 10000 11000        CD14    32.9785173324   0       CD4     41.7117152603   0       0.199805206526  1.0     0.338926100945  1.26481475319   0       OK                                                                                   
   chr1 14000 15000        CD14    9.32978709019   0       CD4     9.31489982941   0       1.0     1.0     -0.00230390372974       0.998404330063  0       OK                                                                                   
   chr1 15000 16000        CD14    9.04603350905   0       CD4     9.01484414416   0       1.0     1.0     -0.00498279072069       0.996552150193  0       OK                                                                                   
   chr1 16000 17000        CD14    0.457565479197  0       CD4     0.14910378845   0       0.677265200643  1.0     -1.61766129852  0.325863281276  0       OK                                                                                   

The second and third window would be merged, as 
   1. Their methylation levels are within 10% of each other.
   2. They are both not differentially methylated.

It aggregates the following:

   * mean values: average
   * std values: max
   * pvalue: max
   * qvalue: max
   * fold: min/max (depending on enrichment/depletion)
   * l2fold: min/max (depending on enrichment/depletion)

The analysis outputs bed files with intervals that are
potentially activated in one of the conditions. Windows
with a positive fold change are collected in the ``treatment``,
while windows with a negative fold change are collected in the
``control``.

For methylation analysis, it might be more interesting
to report windows that are depleted (instead of enriched)
of signal. Thus, if the option ``--invert`` is given,
windows with a negative l2fold change are labeled ``treatment``.
Less methylation means that this region is "active" in the
``treatment`` condition.

Note that the input is assumed to be sorted by coordinate.

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help
 
for command line help.


Command line options
--------------------

'''

import os
import sys
import re
import optparse
import collections

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

DATA = collections.namedtuple( "DATA", "contig start end treatment_name  treatment_mean  treatment_std   control_name    control_mean    control_std     pvalue  qvalue  l2fold  fold    significant     status nintervals")

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-o", "--min-overlap", dest="min_overlap", type="int",
                      help="minimum overlap"  )

    parser.add_option("-w", "--pattern-window", dest="pattern_window", type="string",
                      help="regular expression to extract window coordinates from test id [%default]"  )

    parser.add_option("-i", "--invert", dest="invert", action = "store_true",
                      help="invert treatment/control such that significant windows in control are reported as treatment [%default]"  )

    parser.set_defaults( min_overlap = 10,
                         invert = False,
                         pattern_window = "(\S+):(\d+)-(\d+)"),
    
    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    outfiles = IOTools.FilePool( options.output_filename_pattern )
    
    if options.invert:
        test_f = lambda l2fold: l2fold < 0
    else:
        test_f = lambda l2fold: l2fold > 0

    def read():

        rx_window = re.compile(options.pattern_window)
        # filter any of the DESeq/EdgeR message that end up at the top of the output file
        keep = False
        invert = False
        for line in options.stdin:

            if line.startswith("test_id"): 
                keep = True
                data = line[:-1].split( "\t" )
                # replace test_id with contig, start, end
                if data[1].startswith("control"): invert = True
                continue

            if line.startswith("#"): continue

            if not keep: 
                continue

            try:
                (test_id,
                 a_name, a_mean, a_std,
                 b_name, b_mean, b_std, 
                 pvalue, qvalue, l2fold, fold,
                 significant, status) = line[:-1].split("\t")
            except ValueError:
                E.warn('parsing error in line %s' % line )
                continue

            contig, start, end = rx_window.match(test_id ).groups()
            significant = int(significant)
            start, end = map( int, (start, end ) )
            pvalue, qvalue, l2fold, fold = map( float, (pvalue, qvalue, l2fold, fold) )
            (a_mean, a_std, b_mean, b_std) = \
                map( float, (a_mean, a_std, b_mean, b_std) )

            if invert: 
                a_name, b_name = b_name, a_name
                a_mean, b_mean = b_mean, a_mean
                a_std, b_std = b_std, a_std

            yield DATA._make( (contig, start, end,
                               a_name, a_mean, a_std,
                               b_name, b_mean, b_std,     
                               pvalue, qvalue, l2fold, fold,
                               significant, status,
                               0) )
            
            
    def grouper( data, distance = 10 ):

        last = data.next()
        entries = [last]
        
        while 1:
            d = data.next()
            if d == None: break
            if d.contig == last.contig and d.start < last.start:
                raise ValueError( "error not sorted by start" )

            if ( (d.contig != last.contig) or
                 (d.start - last.end > distance) or
                 (d.status != last.status) or
                 (d.significant != last.significant) or
                 (d.l2fold * last.l2fold < 0) ):
                yield entries
                entries = []
             
            entries.append( d )
            last = d

        yield entries        

    counter = E.Counter()

    options.stdout.write( "\t".join( DATA._fields ) + "\n" )

    # set of all sample names - used to create empty files
    samples = set()

    # need to sort by coordinate
    all_data = list( read() )
    all_data.sort( key = lambda x: (x.contig, x.start) )

    for group in grouper( iter(all_data), distance = options.min_overlap ):

        start, end = group[0].start, group[-1].end
        assert start < end, 'start > end: %s' % str(group)
        n = float(len(group))
        counter.input += n

        g = group[0]

        if g.l2fold < 0:
            l2fold = max( [x.l2fold for x in group ] )
            fold = max( [x.fold for x in group ] )
        else:
            l2fold = min( [x.l2fold for x in group ] )
            fold = min( [x.fold for x in group ] )

        outdata = DATA._make( ( g.contig, start, end,
                                g.treatment_name, 
                                sum( [x.treatment_mean for x in group ] ) / n,
                                max( [x.treatment_std for x in group ] ),
                                g.control_name, 
                                sum( [x.control_mean for x in group ] ) / n,
                                max( [x.control_std for x in group ] ),
                                max( [x.pvalue for x in group ] ),
                                max( [x.qvalue for x in group ] ),
                                l2fold,
                                fold,
                                g.significant,
                                g.status,
                                int(n)) )
                              
        samples.add( g.treatment_name )
        samples.add( g.control_name )
        if g.significant:
            if test_f(g.l2fold):
                # treatment lower methylation than control
                outfiles.write( g.treatment_name, "%s\t%i\t%i\t%s\t%f\n" % (g.contig, g.start, g.end, 
                                                                            g.treatment_name,
                                                                            sum( [x.treatment_mean for x in group ] ) / n ) )
            
            else:
                outfiles.write( g.control_name, "%s\t%i\t%i\t%s\t%f\n" % (g.contig, g.start, g.end, 
                                                                          g.control_name,
                                                                          sum( [x.control_mean for x in group ] ) / n ) )
                
        options.stdout.write("\t".join( map(str, outdata) ) + "\n" )

        counter.output += 1

    for sample in samples:
        outfiles.write( sample, "")

    outfiles.close()
    E.info( "%s" % counter )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#   Gene prediction pipeline 
#
#   $Id: Regions.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
Regions.py - helper functions for working with genomic segments
===============================================================

Version: $Id: Regions.py 2781 2009-09-10 11:33:14Z andreas $
"""

import os, string, re, bisect

class RegionFilter:
    """Filter class based on regions."""

    def __init__(self):
        
        self.mRegions = {}
    
    def readFromFile( self, infile, ignore_strand = False ):
        """read regions from a file."""
    
        self.mForwardRegions = {}
        self.mReverseRegions = {}
        self.mRegions = []
        self.mIgnoreStrand = ignore_strand
        n = 0
        for line in infile:
            if line[0] == "#": continue

            token, sbjct_token, sbjct_strand, sbjct_from, sbjct_to = line[:-1].split("\t")[:5]

            if ignore_strand:
                key = sbjct_token
            else:
                key = "%s-%s" % (sbjct_token, sbjct_strand)
                
            if key not in self.mForwardRegions:
                self.mForwardRegions[key] = []
                self.mReverseRegions[key] = []
                
            self.mForwardRegions[key].append( (int(sbjct_from), n) )
            self.mReverseRegions[key].append( (int(sbjct_to), n) )            
            self.mRegions.append( (token, sbjct_from, sbjct_to) )
            n += 1

        for k, v in self.mForwardRegions.items():
            v.sort()
            self.mForwardRegions[k] = ( map(lambda x: x[0], v),
                                        map(lambda x: x[1], v) )
                                       
        for k, v in self.mReverseRegions.items():
            v.sort()
            self.mReverseRegions[k] = ( map(lambda x: x[0], v),
                                        map(lambda x: x[1], v) )
                                       

    def getOverlaps( self, sbjct_token, sbjct_strand, sbjct_from, sbjct_to):
        """return overlapping regions with region."""

        if self.mIgnoreStrand:
            key = sbjct_token
        else:
            key = "%s-%s" % (sbjct_token, sbjct_strand)
        
        if key not in self.mForwardRegions: return []

        ## index of intervalls starting after sbjct_to
        index_from = bisect.bisect_right( self.mForwardRegions[key][0], sbjct_to - 1)
        ## index of intervalls stopping before sbjct_from
        index_to   = bisect.bisect_left( self.mReverseRegions[key][0], sbjct_from + 1)
        
        set1 = set(self.mForwardRegions[key][1][:index_from])
        set2 = set(self.mReverseRegions[key][1][index_to:])

        overlaps = set1.intersection(set2)
        
#         for o in overlaps:
#             print "overlaps", overlaps
            
#             print "set1: regions not starting after %i" % sbjct_to
#             for x in set1:
#                 print self.mRegions[x]
                
#             print "set1: regions not stopping before %i" % sbjct_from
#             for x in set2:
#                 print self.mRegions[x]                

#             for x in overlaps:
#                 print sbjct_token, sbjct_strand, sbjct_from, sbjct_to, "overlaps with", self.mRegions[x]

        return overlaps



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
merge_tables.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python merge_tables.py --help

Type::

   python merge_tables.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import time

"""merge two tables with the same number of lines
"""

import CGAT.Experiment as E

import CGAT.WrapperAdaptiveCAI as WrapperAdaptiveCAI
import numpy

parser = E.OptionParser( version = "%prog version: $Id: merge_tables.py 2782 2009-09-10 11:40:29Z andreas $")

if __name__ == "__main__":

    parser.add_option("-t", "--table", dest="tables", type="string",
                      help="tables to merge.",
                      action="append")


    parser.set_defaults( \
        tables = [])

    (options, args) = E.Start( parser )

    if len(options.tables) < 1:
        raise "please specify at least one table."

    files = []
    for t in options.tables:
        files.append( open( t, "r"))

    while 1:
        frags = []
        
        stop = False
        
        for f in files:
            l = f.readline()
            if not l:
                stop = True
                break
            frags.append( l[:-1] )

        if stop: break

        print string.join( frags, "\t" )
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
species_map2species_map.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Convert species ids from a species_map file to names provided from 
genome files.

Usage
-----

Example::

   python species_map2species_map.py --help

Type::

   python species_map2species_map.py --help

for command line help.

Documentation
-------------

This is a script with a very specific purpose. Read names that are sampled from
many genomes using GemReads.py are by default converted into substrings of the 
original strings. For sampling contigs using contigs2random_sample.py we require
a species map file with names that correspond to the genomes being simulated.
Therefore this script converts names back to original genome style names using
gi number as the common identifier.

Reads from stdin and outputs to stdout.

Code
----

'''

import os
import sys
import re
import optparse
import CGAT.FastaIterator as FastaIterator
import glob
import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-dir", dest="genome_dir", type="string",
                      help="supply help"  )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    contigs_map = {}
    for genome in glob.glob(os.path.join(options.genome_dir, "*")):
        for fasta in FastaIterator.iterate(IOTools.openFile(genome)):
            identifier = fasta.title.split("|")
            gi = identifier[1]
            contigs_map[gi] = fasta.title

    for line in options.stdin.readlines():
        data = line[:-1].split("\t")
        gi = data[1]
        assert gi in contigs_map, "cannot find genome with id gi|%s in genomes directory" % gi 
            
        options.stdout.write("%s\t%s\n" % (data[0], contigs_map[gi]))

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fastq2fastq.py - manipulate fastq files
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script performs manipulations on :term:`fastq` formatted files. For example
it can be used to change the quality score format or sample a subset of reads.

Usage
-----

Example::

   cat in.fastq.1 | python fastq2fastq.py --sample 0.5 --pair in.fastq.2 --outfile-pair out.fastq.2  > out.fastq.1

In this example we randomly sample 50% of reads from paired data provided in two files.

Type::

   python fastq2fastq.py --help

for command line help.


Code
----

'''

import os
import sys
import re
import optparse
import math
import random
import itertools
import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import CGAT.Fastq as Fastq

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-f", "--change-format", dest="change_format", type="choice",
                      choices = ('sanger', 'solexa', 'phred64', 'integer' ),
                      help="guess quality score format and set quality scores to format [default=%default]."  )

    parser.add_option( "--guess-format", dest="guess_format", type="choice",
                      choices = ('sanger', 'solexa', 'phred64', 'integer' ),
                      help="quality score format to assume if ambiguous [default=%default]."  )

    parser.add_option( "--sample", dest="sample", type="float",
                       help="sample a proportion of reads [default=%default]."  )

    parser.add_option( "--pair", dest="pair", type="string",
                       help="if data is paired, filename with second pair. "
                       "Implemented for sampling [default=%default]."  )

    parser.add_option( "--outfile-pair", dest="outfile_pair", type="string",
                       help="if data is paired, filename for second pair. "
                       "Implemented for sampling [default=%default]."  )

    parser.add_option( "--uniq", dest="uniq", action="store_true",
                       help="remove duplicate reads (by name) [default=%default]."  )

    parser.add_option( "--apply", dest="apply", type="string",
                       help="apply a filter to fastq file (taking only reads in filename) [default=%default]."  )

    parser.add_option( "--trim3", dest="trim3", type="int",
                       help="trim # bases from 3' end [default=%default]."  )

    parser.add_option( "--sort", dest="sort", action="store_true",
                       help="sort fastq by sequence id [default=%default]."  )


    parser.set_defaults(
        change_format = None,
        guess_format = None,
        sample = None,
        trim3 = None,
        pair = None,
        apply = None,
        uniq = False,
        outfile_pair = None,
        sort = None)

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    c = E.Counter()

    if options.change_format:
        for record in Fastq.iterate_convert( options.stdin, 
                                             format = options.change_format,
                                             guess = options.guess_format ):
            c.input += 1
            options.stdout.write( "%s\n" % record )
            c.output += 1
                                                                   
    elif options.sample:
        sample_threshold = min( 1.0, options.sample)
        
        if options.pair:
            if not options.outfile_pair:
                raise ValueError( "please specify output filename for second pair (--outfile-pair)")

            outfile1 = options.stdout
            outfile2 = IOTools.openFile( options.outfile_pair, "w" )
            
            for record1, record2 in itertools.izip( Fastq.iterate( options.stdin ), Fastq.iterate( IOTools.openFile( options.pair) ) ):
                c.input += 1
                if random.random() <= sample_threshold:
                    c.output += 1
                    outfile1.write( "%s\n" % record1 )
                    outfile2.write( "%s\n" % record2 )

        for record in Fastq.iterate( options.stdin ):
            c.input += 1
            if random.random() <= sample_threshold:
                c.output += 1
                options.stdout.write( "%s\n" % record )

    elif options.apply:
        ids = set(IOTools.readList( IOTools.openFile( options.apply ) ))
        
        for record in Fastq.iterate( options.stdin ):
            c.input += 1
            if re.sub(" .*", "", record.identifier).strip() in ids:
                c.output += 1
                options.stdout.write( "%s\n" % record )

    elif options.trim3:
        trim3 = options.trim3
        for record in Fastq.iterate( options.stdin ):
            c.input += 1
            record.trim( trim3 )
            options.stdout.write( "%s\n" % record )
            c.output += 1
            
    elif options.uniq:
        keys = set()
        for record in Fastq.iterate( options.stdin ):
            c.input += 1
            if record.identifier in keys: continue
            else: keys.add( record.identifier )
            options.stdout.write( "%s\n" % record )
            c.output += 1
        
    # Need to change this to incorporate both pairs
    elif options.sort:
        if not options.pair:
            # This is quicker for a single fastq file
            statement = "paste - - - - | sort -k1,1 -t ' ' | tr '\t' '\n'"
            os.system(statement)
        else:
            if not options.outfile_pair:
                raise ValueError( "please specify output filename for second pair (--outfile-pair)")
            
            entries1 = {}
            entries2 = {}
            for record1, record2 in itertools.izip( Fastq.iterate( options.stdin ), Fastq.iterate( IOTools.openFile( options.pair) ) ):
                entries1[record1.identifier[:-2]] = (record1.seq, record1.quals)
                entries2[record2.identifier[:-2]] = (record2.seq, record2.quals)
            
            outfile1 = options.stdout
            outfile2 = IOTools.openFile(options.outfile_pair, "w")
            assert len(set(entries1.keys()).intersection(set(entries2.keys()))) == len(entries1), """paired files do not contain the same reads
                                                                                                     need to reconcile files"""
            for entry in sorted(entries1):
                outfile1.write("@%s/1\n%s\n+\n%s\n" % (entry, entries1[entry][0], entries1[entry][1]))
                outfile2.write("@%s/2\n%s\n+\n%s\n" % (entry, entries2[entry][0], entries2[entry][1]))

    ## write footer and output benchmark information.
    E.info( "%s" % str(c) )
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



"""
Wiggle.py - Support for the `wiggle`_ format used by `ucsc`_.
=============================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

.. _wiggle: http://genome.ucsc.edu/FAQ/FAQformat.html#format5
.. _ucsc: ??
"""

# I had to re-implement the reader, so that random access would work.


import psyco_full
import numpy
from bx.misc.seekbzip2 import SeekableBzip2File
from bx import interval_index_file
import bx.wiggle



class WiggleIndexedAccess( interval_index_file.AbstractIndexedAccess ):
    """
    Indexed access to a wiggle file.
    """
    def get( self, src, start, end, dtype = numpy.float ):
        """return a list of numpy arrays.
        
        Each entry in the list is a tuple (pos, array),
        where pos refers to the residue number of the 
        first position in pos.

        dtype: numpy datatype

        """
        intersections = self.indexes.find( src, start, end ) 
        intersections.sort()
        result = []
        for istart, iend, ival in intersections:
            reader = self.get_at_offset( ival ) 
            xstart=max( istart,start)
            xend=min(end,iend)
            a = numpy.zeros( (min(end,iend)-xstart,), dtype )
            for chr, pos, value in reader:
                ## stay within current contiguous block
                if chr != src or pos >= xend: break
                if pos < start: continue
                a[pos-xstart] = value
            
            result.append( (xstart, a) )
        return result

    def read_at_current_offset( self, file, **kwargs ):
        """
        Read the MAF block at the current position in `file` and return an
        instance of `Alignment`.
        """
        return bx.wiggle.Reader( file, **kwargs )

class WiggleMultiIndexedAccess( interval_index_file.AbstractMultiIndexedAccess ):
    """
    Indexed access to multiple MAF files.
    """
    indexed_access_class = WiggleIndexedAccess

    def get( self, src, start, end, dtype = numpy.float ):
        blocks = []
        for index in self.indexes: 
            blocks.extend( index.get( src, start, end, dtype ) ) 
        return blocks 




#! /bin/env python
################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
blast2table.py - output tabular results from BLAST/PSIBLAST runs
================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

A simple BLAST parser. This parser requires BLAST+. BLAST+ should be run with the
following output options::

   -outfmt "6 qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq"

The option *iterative* describes how results from iterative 
searches (PSIBLAST) should be reported. If set, it will automatically
add an additional field iteration:

all
   Results of all iterations are output.

last
   Results of the last iteration only are output.

first
   For each alignment between query and sbjct, the first 
   alignment is output.

.. note::

   Coordinates are 0-based, open-closed.

The option *alignment_format* selects the alignment format:

emissions

   The alignment is output as a string of positive and negative 
   integers for both query and sbjct sequence. Positive integers
   are emissions, negative integers denotes gaps.

blocks
   Block based alignment format. The alignment is reported as
   three ',' separated lists of block sizes and block positions
   in the query and sbjct sequence. Coordinates are relative to
   the alignment start.

Usage
-----

Type::

   python blast2table.py --help

for command line help.

Blast parsing
+++++++++++++

The following command outputs blast results using the pairsdb alignment format::

   blastp -query <(head -n 10 nrdb.fasta ) -db nrdb -outfmt "6 qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq" |
   python python blast2table.py --output-format=emissions

The following command outputs blast results using the pairsdb alignment format::

   blastp -query <(head -n 10 nrdb.fasta ) -db nrdb -outfmt "6 qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq" |
   python python blast2table.py --output-format=blocks

Psiblast parsing
++++++++++++++++

The following command outputs alignments from all iterations::

   psiblast -query <(head -n 2 nrdb.fasta ) -db nrdb -outfmt "6 qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq" -num_iterations=5
   python blast2table.py --alignment-format=blocks --iterations=all

The following command outputs alignments from the last iteration only::

   psiblast -query <(head -n 2 nrdb.fasta ) -db nrdb -outfmt "6 qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq" -num_iterations=5
   python blast2table.py --alignment-format=blocks --iterations=all

The following command outputs alignments for the first iteration that they are found::

   psiblast -query <(head -n 2 nrdb.fasta ) -db nrdb -outfmt "6 qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq" -num_iterations=5
   python blast2table.py --alignment-format=blocks --iterations=first

'''

import os
import sys
import re
import optparse
import collections
import CGAT.Experiment as E

BlastResult = collections.namedtuple( "blastresult",
                                      "qseqid qstart qend sseqid sstart send evalue bitscore pident score qseq sseq" )

class Output( object ):

    def __init__(self, line ):

        data = line[:-1].split("\t")
        try:
            self.r = BlastResult._make( data )
        except TypeError:
            raise ValueError( "parsing error in line: '%s'" % line[:-1])
        
        # define some shortcuts for fields used in processing later
        self.query = self.r.qseqid
        self.sbjct = self.r.sseqid
        self.bitscore = float(self.r.bitscore)
        self.iteration = None

class OutputEmissions( Output ):
    '''output blast results in the legacy pairsdb emissions format.'''

    header = ["query_nid", "sbjct_nid", "evalue",
              "query_start", "query_end", "query_ali",
              "sbjct_start", "sbjct_end", "sbjct_ali",
              "bitscore", "pid" ]
    
    def __str__( self ):
        
        def seq2ali( seq ):
            ali = seq.split( "-" )
            result = []
            x = 0
            while x < len(ali)-1:
                result.append( "+%i" % len(ali[x] ) )
                x += 1
                g = x
                while ali[x] == "": x += 1
                result.append( "-%i" % (x-g+1) ) 
            result.append( "+%i" % len(ali[-1] ) )
            return "".join(result)

        return "\t".join( ( self.r.qseqid,
                            self.r.sseqid,
                            self.r.evalue,
                            str(int(self.r.qstart) -1),
                            self.r.qend,
                            seq2ali( self.r.qseq ),
                            str(int(self.r.sstart) -1),
                            self.r.send,
                            seq2ali( self.r.sseq ),
                            self.r.bitscore,
                            self.r.pident ) )

class OutputBlocks( Output ):
    '''output blast alignments as blocks.'''

    header = ["query_nid", "sbjct_nid", "evalue",
              "query_start", "query_end", 
              "sbjct_start", "sbjct_end", 
              "block_sizes", "query_starts", "query_ends",
              "bitscore", "pid" ]
    
    def __str__( self ):

        block_sizes, query_starts, sbjct_starts = [], [], []

        in_block = True
        block_size = 0
        qc, sc = 0, 0
        query_starts.append( qc )
        sbjct_starts.append( sc )
        
        for qa, sa in zip( self.r.qseq, self.r.sseq ):
            qisgap = qa == "-"
            sisgap = sa == "-"
            if qisgap or sisgap:
                if in_block: block_sizes.append( block_size )
                in_block = False
            else:
                if not in_block:
                    # start of new block
                    query_starts.append( qc )
                    sbjct_starts.append( sc )
                    block_size = 0
                block_size += 1
                in_block = True

            if not qisgap: qc += 1
            if not sisgap: sc += 1

        block_sizes.append( block_size)

        return "\t".join( ( self.r.qseqid,
                            self.r.sseqid,
                            self.r.evalue,
                            str(int(self.r.qstart) -1),
                            self.r.qend,
                            str(int(self.r.sstart) -1),
                            self.r.send,
                            ",".join( map(str, block_sizes) ),
                            ",".join( map(str, query_starts) ),
                            ",".join( map(str, sbjct_starts) ),
                            self.r.bitscore,
                            self.r.pident ) )

def addIterations( group ):
    '''for each pair of query and sbjct add the
    iteration.

    There are no markers in psiblast output
    for the end of an iteration. Hence,
    this method uses as a proxy the bitscore.

    returns a dictionary and the number of iterations
    '''

    query2sbjct_pairs = collections.defaultdict( list )
    last_score = group[0].bitscore
    iteration = 1
    for r in group:
        if r.bitscore > last_score:
            iteration += 1
        r.iteration = iteration
        query2sbjct_pairs[ (r.query, r.sbjct) ].append( iteration )
        last_score = r.bitscore

    # normalize
    normed = {}
    for pair, iterations in query2sbjct_pairs.iteritems():
        normed[pair] = sorted(list( set( iterations )))
    return normed, iteration

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id$", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-f", "--alignment-format", dest="alignment_format", type="choice",
                      choices = ("emissions", "blocks" ),
                      help="output format options [default=%default]."  )

    parser.add_option( "--no-header", dest="with_header", action="store_false",
                       help="skip output of header [default=%default]."  )

    parser.add_option("-i", "--iterations", dest="iterations", type="choice",
                      choices = ("last", "all", "first", None ),
                      help="output for iterative searches [default=%default]."  )


    parser.set_defaults(
        alignment_format = "emissions",
        with_header = True,
        iterations = None,
        )

    (options, args) = parser.parse_args() 

    if options.alignment_format == "emissions":
        outer = OutputEmissions

    elif options.alignment_format == "blocks":
        outer = OutputBlocks

    if options.with_header:
        header = outer.header[:]
        if options.iterations != None: header.append( "iteration" )
        sys.stdout.write( "\t".join( header) + "\n" )

    def grouper( infile ):
        
        group = []
        
        query = None
        for line in infile:
            if line.startswith("#"): continue
            if not line.strip(): continue
            if line.startswith("Search has CONVERGED!"):
                sys.stdout.write("# %s" % line)
                continue

            r = outer( line )
            if r.query != query:
                if query: yield group
                query, group = r.query, []

            group.append( r )

        if query: yield group

    for group in grouper( sys.stdin ):
        if options.iterations == None:
            for r in group:
                sys.stdout.write( str(r) + "\n" )
        else:
            query2sbjct_pairs, max_iterations = addIterations( group )
            if options.iterations == "all":
                filtered = group
            elif options.iterations == "last":
                filtered = ( r for r in group if r.iteration == max_iterations )
            elif options.iterations == "first":
                output_pairs = {}
                for r in group:
                    key = (r.query,r.sbjct)
                    if key not in output_pairs: output_pairs[key] = r.iteration
                filtered = (r for r in group if r.iteration <= output_pairs[(r.query,r.sbjct)] )
                        
            for r in filtered:
                sys.stdout.write( "%s\t%i\n" % (str(r), r.iteration) )

if __name__ == "__main__":
    sys.exit( main( sys.argv) )
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
sparse2full.py - convert a sparse matrix to adjacency matrix
============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert a sparse matrix to adjacency matrix and vice versa.

Usage
-----

Example::

   python sparse2full.py --help

Type::

   python sparse2full.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import optparse
import string
import re
import random

import CGAT.Experiment as E

def CountElements( matrix, default_value ):
    """count elements that are of default value and those that aren't."""

    ndefault = 0
    for x in matrix:
        for y in x:
            if default_value == y:
                ndefault += 1
                
    nfound = reduce( lambda x,y : x + y, [ len(x) for x in matrix] ) - ndefault
    return ndefault, nfound

def Sparse2Matrix( outfile, matrix_id, lines, options, in_map_token2row = {}, in_map_token2col = {}):

    # remove comments
    lines = filter(lambda x:x[0] != "#" and len(x[:-1]) > 0, lines)

    if len(lines) == 0: raise IOError("no input")

    # forget about titles
    v = lines[0][:-1].split("\t")[2]
    if v not in ("na", "NaN"):
        try:
            v = float(v)
        except ValueError:
            del lines[0]

    if in_map_token2row:
        map_token2row = in_map_token2row
    else:
        map_token2row = {}

    if in_map_token2col:
        map_token2col = in_map_token2col
    else:
        map_token2col = {}

    row_converter, col_converter = str, str
    if options.format == "string":

        has_row_names = len(map_token2row) > 0
        has_col_names = len(map_token2col) > 0

        ## if either row/column names are not given:
        if not map_token2row or not map_token2col:
            row_tokens = map(lambda x: string.split(x[:-1], "\t")[0], lines )
            col_tokens = map(lambda x: string.split(x[:-1], "\t")[1], lines )

            if options.input_format == "row-col-weight-weight":
                # merge row and col tokens
                row_tokens.extend(col_tokens)
                col_tokens = row_tokens
            
            if options.is_numeric:
                try:
                    row_tokens = map(int, row_tokens)
                    row_converter = int
                except ValueError:
                    row_tokens = map(float, row_tokens)
                    row_converter = float

                try:
                    col_tokens = map(int, col_tokens)                    
                    col_converter = int
                except ValueError:
                    col_tokens = map(float, col_tokens)                    
                    col_converter = float

            row_tokens.sort()
            col_tokens.sort()

            if not has_row_names:
                for row_token in row_tokens:
                    if row_token not in map_token2row:
                        map_token2row[row_token] = len(map_token2row)
            if not has_col_names:
                for col_token in col_tokens:
                    if col_token not in map_token2col:
                        map_token2col[col_token] = len(map_token2col)

        if not options.asymmetric:
            for col_token in map_token2col.keys():
                if col_token not in map_token2row:
                    map_token2row[col_token] = len(map_token2row)            
            map_token2col = map_token2row

            

        matrix = [ [ options.default for j in range(len(map_token2col))] for i in range(len(map_token2row)) ]

        if len(map_token2col) == len(map_token2row):
            for j in range(len(map_token2col)):
                matrix[j][j] = options.default_diagonal

        ## replicate counts
        replicates = [ [ 0 for j in range(len(map_token2col))] for i in range(len(map_token2row)) ]
        
        if options.input_format == "row-col-weight":
            for line in lines:
                row_token, col_token, weight = string.split(line[:-1], "\t")[:3]
                row_token, col_token = row_converter( row_token ), col_converter(col_token)
                matrix[map_token2row[row_token]][map_token2col[col_token]] = weight
                replicates[map_token2row[row_token]][map_token2col[col_token]] += 1                
                if not options.asymmetric:
                    matrix[map_token2col[col_token]][map_token2row[row_token]] = weight
                    replicates[map_token2col[col_token]][map_token2row[row_token]] += 1
                    
        elif options.input_format == "row-col-weight-replicates":
            for line in lines:
                row_token, col_token, weight, n = string.split(line[:-1], "\t")[:4]
                matrix[map_token2row[row_token]][map_token2col[col_token]] = weight
                replicates[map_token2row[row_token]][map_token2col[col_token]] = int(n)
                if not options.asymmetric:
                    matrix[map_token2col[col_token]][map_token2row[row_token]] = weight
                    replicates[map_token2col[col_token]][map_token2row[row_token]] = int(n)                     

        elif options.input_format == "row-col-weight-weight":
            for line in lines:
                row_token, col_token, weight1, weight2 = string.split(line[:-1], "\t")[:4]
                matrix[map_token2row[row_token]][map_token2col[col_token]] = weight1
                matrix[map_token2col[col_token]][map_token2row[row_token]] = weight2
                replicates[map_token2row[row_token]][map_token2col[col_token]] += 1                
                replicates[map_token2row[col_token]][map_token2col[row_token]] += 1                
                    
        col_tokens = map_token2col.items()
        col_tokens.sort( lambda x,y: cmp(x[1],y[1]))
        row_tokens = map_token2row.items()
        row_tokens.sort( lambda x,y: cmp(x[1],y[1]))

        ndefault, nfound = CountElements( matrix, options.default )

        ## Apply filtering to decide whether matrix should be output

        ## 1. Filter by number of rows/columns
        if (options.filter_numrows and len(row_tokens) != options.filter_numrows) or\
               (options.filter_numcols and len(col_tokens) != options.filter_numcols):
            if options.loglevel >= 1:
                options.stdlog.write( "# id=%s, nrows=%i, ncols=%i, nfound=%i, ndefault=%i\n" % ( matrix_id, len(row_tokens), len(col_tokens), nfound, ndefault) )
            return False

        if options.output_format == "square":

            for col_token, index in col_tokens:
                outfile.write("\t%s" % col_token)
            outfile.write("\n")

            for row in range(len(matrix)):
                outfile.write("%s\t%s\n" % (row_tokens[row][0], string.join(matrix[row], "\t")))

        elif options.output_format == "phylip":

            if len(row_tokens) != len(col_tokens):
                raise ValueError, "phylip needs symmetric matrices."
            
            outfile.write ("%i\n" % len(row_tokens) )
            
            for row in range(len(matrix)):
                outfile.write("%-10s\t%s\n" % (row_tokens[row][0], " ".join(map(lambda x: "  %10s" % x, matrix[row] ))))

        elif options.output_format == "phylip-replicates":

            if len(row_tokens) != len(col_tokens):
                raise ValueError, "phylip needs symmetric matrices."
            
            outfile.write ("%i\n" % len(row_tokens) )

            for row in range(len(matrix)):
                outfile.write("%-10s" % row_tokens[row][0])
                rr = matrix[row]
                re = replicates[row]
                for c in range(len(matrix[row])):
                    outfile.write(" %10s %i" % (rr[c], re[c]))
                outfile.write("\n")
    else:
        if not options.row_names or not options.col_names:
            raise "Please specify row and column range."

        row_range = eval(options.row_names)
        col_range = eval(options.col_names)

        map_row = {}
        map_col = {}
        for x in range(len(row_range)): map_row[row_range[x]] = x
        for x in range(len(col_range)): map_col[col_range[x]] = x

        matrix = [ [ options.default for j in col_range] for i in row_range ]
        for line in lines:
            row_token, col_token, weight = string.split(line[:-1], "\t")[:3]

            row_pos = map_row[int(float(row_token))]
            col_pos = map_col[int(float(col_token))]
            # col_pos = int(float(col_token)) - col_range[0]

            if row_pos < 0 or row_pos > row_range[-1]: continue
            if col_pos < 0 or col_pos > col_range[-1]: continue                

            matrix[row_pos][col_pos] = weight

            if not options.asymmetric:
                matrix[col_pos][row_pos] = weight

        for col in col_range:
            outfile.write("\t%i" % col)
        outfile.write("\n")

        ndefault, nfound = CountElements( matrix, options.default )

        for row in range(len(matrix)):
            outfile.write("%i\t%s\n" % (row_range[row], "\t".join(map(str, matrix[row]))))
            
    if options.loglevel >= 1:
        options.stdlog.write( "# id=%s, nrows=%i, ncols=%i, nfound=%i, ndefault=%i\n" % ( matrix_id, len(row_tokens), len(col_tokens), nfound, ndefault) )

    return True

if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: sparse2full.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-m", "--method", dest="methods", type="string",
                      help="""method to use [normalize-by-min-diagonal|normalize-by-column|
                      log|ln|negzero2value|normalize-by-matrix|subtract-matrix|mix-matrix|
                      normalize-by-column-max|normalize-by-row-max|correspondence-analysis|
                      transpose|upper-bound|lower-bound
                      """  )
    
    parser.add_option("-f", "--format", dest="format", type="string",
                      help="format."  )
    
    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=("square", "phylip", "phylip-replicates"),
                      help="output format."  )

    parser.add_option("-i", "--input-format", dest="input_format", type="choice",
                      choices=("row-col-weight", "row-col-weight-replicates", "row-col-weight-weight" ),
                      help="input format."  )
    
    parser.add_option("-a", "--asymmetric", dest="asymmetric", action="store_true",
                      help="matrix is asymmetric."  )

    parser.add_option("-d", "--default", dest="default", type="string",
                      help="default value for missing values."  )

    parser.add_option("-D", "--default-diagonal", dest="default_diagonal", type="string",
                      help="default value for missing values on diagonal."  )

    parser.add_option("-r", "--row-names", dest="row_names", type="string",
                      help="list of row names separated by ','."  )
    
    parser.add_option("-c", "--col-names", dest="col_names", type="string",
                      help="list of col names separated by ','."  )

    parser.add_option( "--file-row-names", dest="file_row_names", type="string",
                      help="filename with row names."  )
    
    parser.add_option( "--file-col-names", dest="file_col_names", type="string",
                      help="filename with col names."  )

    parser.add_option("--full2sparse", dest="full2sparse", action="store_true",
                      help="convert full to sparse matrix."  )

    parser.add_option("--skip-separators", dest="write_separators", action="store_false",
                      help="do not echo separators (starting with >)")

    parser.add_option("-n", "--numeric", dest="is_numeric", action="store_true",
                      help="row and column titles are numeric (for sorting)."  )

    parser.add_option( "--filter-numcols", dest="filter_numcols", type="int",
                      help="only output matrices with # columns."  )

    parser.add_option( "--filter-numrows", dest="filter_numrows", type="int",
                      help="only output matrices with # rows."  )

    parser.add_option( "--filename-map", dest="filename_map", type="string",
                      help="filename with mapping between input and output chunks."  )

    parser.set_defaults(
        default = "0",
        default_diagonal = "0",
        asymmetric = False,
        row_names = None,
        col_names = None,
        file_row_names = None,
        file_col_names = None,
        full2sparse = False,
        format = "string",
        is_numeric = False,
        output_format = "square",
        input_format = "row-col-weight",
        filter_numcols = 0,
        filter_numrows = 0,
        filename_map = None,
        write_separators = True,
        )

    (options, args) = E.Start( parser )

    if options.full2sparse:
        # convert a full matrix to a sparse matrix

        options.stdout.write("row\tcol\tvalue\n" )

        if options.row_names:
            row_tokens = map( lambda x: string.split(x[:-1], "\t")[0], open(options.row_names, "r").readlines())
        else:
            row_tokens = None
            
        col_tokens = None
        
        row = 0
        for line in sys.stdin:
            if line[0] == "#": continue

            data = line[:-1].split("\t")

            row += 1
            if row == 1:
                if not col_tokens:
                    if options.col_names:
                        col_tokens = map( lambda x: string.split(x[:-1], "\t")[0], open(options.col_names, "r").readlines())
                    else:
                        if not row_tokens: del data[0]
                        col_tokens = map(str, data) 
                    continue
                
            if row_tokens:
                row_token = row_tokens[row]
            else:
                row_token = data[0]
                del data[0]
            
            for x in range(len(data)):
                if data[x] != options.default:
                    options.stdout.write("%s\t%s\t%s\n" % (row_token, col_tokens[x], data[x] ) )

    else:
        nskipped = 0
        
        if options.filename_map:
            outfile_map = open(options.filename_map, "w")
            outfile_map.write("## Map between matrices in input file and output matrices.\nnew\told\n" )
            
        # convert a sparse matrix to a full matrix        
        lines = filter( lambda x: x[0] != "#", sys.stdin.readlines())

        chunks = filter( lambda x: lines[x][0] == ">", range(len(lines)) )

        if not chunks:
            chunks = [-1]
            options.write_separators = False
            
        chunks.append( len(lines) )

        if options.loglevel >= 2:
            print "# processing chunks:", chunks

        map_token2row = {}
        map_token2col = {}

        if options.file_row_names:
            row_tokens = map( lambda x: string.split(x[:-1], "\t")[0], open(options.file_row_names, "r").readlines())
            for row_token in row_tokens:
                map_token2row[row_token] = len(map_token2row)

        if options.row_names:
            for x in options.row_names.split(","):
                map_token2row[x] = len(map_token2row)

        if options.file_col_names:
            col_tokens = map( lambda x: string.split(x[:-1], "\t")[0], open(options.file_col_names, "r").readlines())
            for col_token in col_tokens:
                map_token2col[col_token] = len(map_token2col)

        if options.col_names:
            for x in options.col_names.split(","):
                map_token2col[x] = len(map_token2col)

        if not options.asymmetric:
            if map_token2row and not map_token2col:
                map_token2col = map_token2row
            elif map_token2col and not map_token2row:
                map_token2row = map_token2col

        noutput = 0
        
        for x in range(len(chunks) -1 ):

            if options.write_separators:
                options.stdout.write( lines[chunks[x]] )
                    
            output = Sparse2Matrix( options.stdout, str(x+1), lines[chunks[x]+1:chunks[x+1]],
                                    options,
                                    map_token2row, map_token2col )

            if not output:
                nskipped += 1
            else:
                noutput += 1
                if options.filename_map:
                    outfile_map.write( "%i\t%i\n" % (noutput, x+1))

        if options.filename_map:
            outfile_map.close()
                
    E.Stop()



####
####
##
##
## Copyright (C) 2008 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: bed2bed.py 2861 2010-02-23 17:36:32Z andreas $
##
##
####
####
'''
wig2bed.py - convert densities to intervals
===========================================

Purpose
-------

define intervals based on densities within a bigwig file.

The script currently implements the following methods (``--method``):

threshold
    output windows that contain values above a certain
    threshold.

std-above-mean
    output windows that are a certain number of standard deviations 
    above the mean. 

multiple-of-mean
    output windows that are a certain times above the mean.

Usage
-----

Bigwig files need to be supplied by the --bigwig-file options.

For example::

    python wig2bed.py --threshold=10 --method=threshold --genome-file=mm10 --bigwig-file=in.bw > out.bed

Command line options
--------------------

'''   

import sys
import re
import string
import optparse
import time
import os
import itertools
import tempfile
import subprocess
import shutil
import collections

import CGAT.Experiment as E
import CGAT.IndexedFasta as IndexedFasta
import CGAT.IOTools as IOTools

import bx
import bx.bbi.bigwig_file

def applyThreshold( infile, fasta, threshold, max_distance = 0 ):
    '''apply threshold to a wig file writing a
    bed-formatted file as output.'''
    
    c = E.Counter()

    for contig, size in fasta.getContigSizes( with_synonyms = False ).items():
        c.contigs += 1

        E.debug( "processing %s" % contig )
        
        iterator = infile.get( contig, 0, size )
        if iterator == None: 
            c.skipped += 1
            E.warn( "skipping %s" % contig )
            continue

        last_start, last_end = -1, 0

        for start, end, value in iterator:
            d = start - last_end
            if (d > 0 or value < threshold): 
                if last_start >= 0:
                    yield contig, last_start, last_end
                    c.intervals += 1
                last_start = -1
            elif last_start < 0 and value >= threshold:
                last_start = start

            last_end = end

        if last_start >= 0:
            yield contig, last_start, end
            c.intervals += 1

        c.output += 1

    E.info( str(c) )

def getBigwigSummary( bigwig_file ):
    '''return summary of bigwig contents.

    This method uses the bigWigInfo UCSC utility
    '''
    
    results = E.run( "bigWigInfo %(bigwig_file)s" % locals(), return_stdout = True )

    data = [ x.split( ":") for x in results.split( "\n" ) if x != "" ]
    fields = [ x[0] for x in data]
    Results = collections.namedtuple( "BigwigInfo", fields )
    def conv( v ):
        return IOTools.str2val( re.sub( ",", "", v.strip()) )

    results = Results( *[ conv(x[1]) for x in data] )
    return results

def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id: bed2bed.py 2861 2010-02-23 17:36:32Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-m", "--method", dest="methods", type="choice", action="append",
                       choices=("threshold", "stddev-above-mean", "multiple-of-mean"),
                       help="method to apply [default=%default]"  )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome."  )

    parser.add_option("-t", "--threshold", dest="threshold", type="float",
                      help="threshold to apply [default=%default]"  )

    parser.add_option("-i", "--bigwig-file", dest="bigwig_file", type="string", metavar="bigwig",
                      help="filename with bigwig information [default=%default]."  )

    parser.add_option("-b", "--bigwig", dest="bigwig", action = "store_true",
                      help="input is bigwig [default=%default]."  )

    parser.set_defaults( methods = [],
                         genome_file = None,
                         threshold = 10,
                         bigwig = False,
                         max_distance = 0)
    
    (options, args) = E.Start( parser, add_pipe_options = True )

    if options.bigwig_file:
        bigwig_file = bx.bbi.bigwig_file.BigWigFile( open(options.bigwig_file ) )
    else:
        bigwig_file = None

    if options.genome_file:
        genome_fasta = IndexedFasta.IndexedFasta( options.genome_file )
        contigs = genome_fasta.getContigSizes()

    for method in options.methods:
        if method ==  "threshold":
            if not contigs: raise ValueError("please supply contig sizes" )
            if not bigwig_file: raise NotImplementedError( "threshold not implemented for wig files")
            processor = applyThreshold( bigwig_file, 
                                        genome_fasta, 
                                        threshold = options.threshold,
                                        max_distance = options.max_distance )
        elif method ==  "stddev-above-mean":
            if not contigs: raise ValueError("please supply contig sizes" )
            if not bigwig_file: raise NotImplementedError( "threshold not implemented for wig files")
            summary = getBigwigSummary( options.bigwig_file )
            threshold = summary.mean + options.threshold * summary.std
            E.info( "applying threshold %f: mean=%f, std=%f" % (threshold, summary.mean, summary.std) )
            processor = applyThreshold( bigwig_file, 
                                        genome_fasta, 
                                        threshold = threshold,
                                        max_distance = options.max_distance )

        elif method ==  "multiple-of-mean":
            if not contigs: raise ValueError("please supply contig sizes" )
            if not bigwig_file: raise NotImplementedError( "threshold not implemented for wig files")
            summary = getBigwigSummary( options.bigwig_file )
            threshold = summary.mean * options.threshold 
            E.info( "applying threshold %f: mean=%f, std=%f" % (threshold, summary.mean, summary.std) )
            processor = applyThreshold( bigwig_file, 
                                        genome_fasta, 
                                        threshold = threshold,
                                        max_distance = options.max_distance )


    outfile = options.stdout
    
    outfile.write( "".join( [ "%s\t%i\t%i\n" % x for x in processor ] ) )
    outfile.write( "\n" )

    E.Stop()


if __name__ == "__main__":
    sys.exit( main( sys.argv ) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
snp2snp.py - manipulate lists of SNPs
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------


Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import glob

import pysam

import CGAT.Experiment as E

def validateSNPs( options, fastafile ):
    '''read SNPs in pileup format.'''

    callers = []

    for filename in options.filename_references:
        for f in glob.glob( filename ):
            callers.append( pysam.SNPCaller( pysam.Samfile( f, "rb" ), fastafile ) )

    if len(callers) == 0: 
        E.warning( "no transcript data available" )
    else:
        E.info( "validating against %i reference files" % (len(callers)))

    c = E.Counter()

    outf = options.stdout

    nfiles = len(callers)

    outf.write( "\t".join( \
            ("contig", "pos", "reference", "genotype", "consensus_quality", "genotype_quality", "mapping_quality", "coverage" ) ) )
    outf.write( "\tstatus\tcalls\tfiltered\tmin_coverage\tmax_coverage\tmin_quality\tmax_quality\t" )
    outf.write( "\t".join( ("genotypes", "consensus_qualities", "genotype_qualities", "mapping_qualities", "coverages" ) ) )
    outf.write( "\n" )

    min_coverage = options.min_coverage

    for line in options.stdin:

        c.input += 1

        if c.input % options.report_step == 0:
            E.info("iteration %i: %s" % (c.input, str(c)))

        data = line[:-1].split()

        contig, pos, reference, genotype, consensus_quality, genotype_quality, mapping_quality, read_depth = data[:8]
        pos, consensus_quality, genotype_quality, mapping_quality, read_depth = \
            map( int, (pos, consensus_quality, genotype_quality, mapping_quality, read_depth))

        if reference == "*":
            # todo: treat indels
            c.skipped += 1
            continue

        # go to 0-based coordinates
        pos -= 1
        calls = []
        for x, caller in enumerate(callers):
            try:
                calls.append( caller.call( contig, pos ) )
            except ValueError, msg:
                continue
        outf.write( "\t".join( map(str, 
                                   (contig, pos, reference, genotype, 
                                    consensus_quality, genotype_quality, mapping_quality, read_depth ) )))

        gt = [ x.genotype for x in calls ]
        cq = [ x.consensus_quality for x in calls]
        sq = [ x.snp_quality for x in calls]
        mq = [ x.mapping_quality for x in calls ]
        co = [ x.coverage for x in calls ]

        # filter calls by quality
        filtered_calls = [ x for x in calls if x.coverage >= min_coverage ] 

        nidentical = len( [x for x in filtered_calls if x.genotype == genotype ] )
        nreference = len( [x for x in filtered_calls if x.genotype == reference ] )
        ncalls = len(filtered_calls)

        if ncalls == 0:
            # no data
            status = "?"
        elif nidentical == ncalls:
            # ok
            status = "O"
        elif nidentical:
            # ambiguous
            status = "A"
        elif nreference == ncalls:
            # wildtype
            status = "W"
        else:
            # conflict
            status = "C"

        c[status] += 1

        outf.write( "\t%s\t%i\t%i" % ( status, len(calls), len(filtered_calls) ) )

        if len(calls) > 1:
            outf.write( "\t%i\t%i\t%i\t%i" % (
                    min( co ), max(co),
                    min( sq ), max(sq) ))
        else:
            outf.write( "\t" + "\t".join( (
                        "".join( map(str,co) ), "".join(map(str,co)),
                        "".join( map(str,sq) ), "".join(map(str,sq))) ) )

        outf.write( "\t%s" % ",".join( map(str, gt ) ) )
        outf.write( "\t%s" % ",".join( map(str, cq ) ) )
        outf.write( "\t%s" % ",".join( map(str, sq ) ) )
        outf.write( "\t%s" % ",".join( map(str, mq ) ) )
        outf.write( "\t%s" % ",".join( map(str, co ) ) )
        outf.write("\n")

        c.output += 1

    return c

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id$", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices = ("validate", ),
                      help="methods to apply [default=%default]."  )

    parser.add_option("-g", "--filename-genome", dest="filename_genome", type="string",
                      help="filename of faidx indexed genome [default=%default]."  )

    parser.add_option("-r", "--filename-reference", dest="filename_references", action="append",
                      help="reference filenames for validation step [default=%default]."  )

    parser.add_option( "--min-coverage", dest="min_coverage", type="int",
                      help="minimum coverage [default=%default]."  )

    parser.set_defaults(
        min_coverage = 0,
        method = None,
        filename_genome = None,
        filename_references = [],
        report_step = 10000,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if options.method == None:
        raise ValueError("please supply a method")

    if options.filename_genome != None:
        fastafile = pysam.Fastafile( options.filename_genome )    
    else:
        fastafile = None

    if options.method == "validate":
        counter = validateSNPs( options, fastafile )

    E.info( "%s" % str(counter) )
    
    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
rnaseq_bams2bam.py - convert mappings against junctions to genomic coordinates
================================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes as input a BAM file resulting from reads mapped against
a junction database and outputs a :term:`bam` formatted file in genomic
coordinates.

The contigs should be of the format 
<chromosome>|<start>|<exon-end>-<exon-start>|<end>|<splice>|<strand>.

<start> - 0-based coordinate of first base
<exon-end> - 0-based coordinate of last base in exon
<exon-start> - 0-based coordinate of first base in exon
<end> - 0-based coordinate of base after last base

Strand can be either ``fwd`` or ``rev``, though sequences in the database
and coordinates are all on the forward strand.

For example ``chr1|1244933|1244982-1245060|1245110|GTAG|fwd`` tranlates to the
intron ``chr1:1244983-1245060`` in python coordinates.

The input bam-file is supposed to be sorted by read. Only the best matches
are output for each read, were best is defined both in terms of number 
of mismatches and number of colour mismatches.

Usage
-----

Example::

   cat input.bam | python rnaseq_junction_bam2bam.py - --log=log > output.bam

Type::

   python rnaseq_junction_bam2bam.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import time
import itertools

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import pysam

import pyximport
pyximport.install(build_in_temp=False)
import _rnaseq_bams2bam

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-t", "--template-bam", dest="filename_genome_bam", type="string",
                       help = "input bam file for header information [%default]" )

    parser.add_option( "-s", "--contig-sizes", dest="filename_contigs", type="string",
                       help = "filename with contig sizes [%default]" )

    parser.add_option( "-o", "--colour", dest="colour_mismatches", action="store_true",
                       help = "mismatches will use colour differences (CM tag) [%default]" )

    parser.add_option( "-i", "--ignore-mismatches", dest="ignore_mismatches", action="store_true",
                       help = "ignore mismatches [%default]" )

    parser.add_option( "-c", "--remove-contigs", dest="remove_contigs", type="string",
                       help = "','-separated list of contigs to remove [%default]" )

    parser.add_option( "-f", "--force", dest="force", action = "store_true",
                       help = "force overwriting of existing files [%default]" )

    parser.add_option( "-u", "--unique", dest="unique", action = "store_true",
                       help = "remove reads not matching uniquely [%default]" )

    parser.set_defaults(
        filename_genome_bam = None,
        filename_gtf = None,
        filename_mismapped = None,
        remove_contigs = None,
        force = False,
        unique = False,
        colour_mismatches = False,
        ignore_mismatches = False,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    genomefile, referencenames, referencelengths = None, None, None

    if options.filename_genome_bam:
        genomefile = pysam.Samfile( options.filename_genome_bam, "rb" )
    elif options.filename_contigs:
        contigs = IOTools.ReadMap( IOTools.openFile( options.filename_contigs ) )
        data = zip( *list( contigs.iteritems() ) )
        referencenames, referencelengths = data[0], map(int, data[1])
    else:
        raise ValueError("please provide either --template-bam or --contig-sizes" )

    infile = pysam.Samfile( "-", "rb" )
    outfile = pysam.Samfile( "-", "wb", template = genomefile,
                             referencenames = referencenames,
                             referencelengths = referencelengths )
    
    if options.colour_mismatches: 
        tag = "CM"
    else:
        tag = "NM"

    nambiguous = 0
    ninput = 0
    nunmapped = 0
    ncigar = 0
    nfull = 0
    noutput = 0

    contig2tid = dict( [ (y,x) for x,y in enumerate( outfile.references ) ] )

    for qname, readgroup in itertools.groupby( infile, lambda x: x.qname ):
        ninput += 1
        reads = list(readgroup)
        if reads[0].is_unmapped:
            nunmapped += 1
            continue

        # filter for best match
        best = min( [ x.opt(tag) for x in reads ] )
        reads = [ x for x in reads if x.opt(tag) == best ]
        if len(reads) > 1:
            nambiguous += 1
            continue
        
        read = reads[0]

        # reject complicated matches (indels, etc)
        # to simplify calculations below.
        if len(read.cigar) > 1:
            ncigar += 1
            continue 
        
        # set NH flag to latest count
        t = dict( read.tags )
        t['NH'] = 1
        read.tags = list(t.iteritems())

        sname = infile.getrname( read.tid )

        contig, first_exon_start, middle, last_exon_end, splice, strand = sname.split("|")
        first_exon_end, last_exon_start = middle.split("-")
        first_exon_start, first_exon_end, last_exon_start, last_exon_end = map(int, (\
                first_exon_start, first_exon_end, last_exon_start, last_exon_end ) )
        first_exon_end += 1

        total = first_exon_end - first_exon_start + last_exon_end - last_exon_start
        first_exon_length = first_exon_end - first_exon_start

        match1 = first_exon_length - read.pos
        intron_length = last_exon_start - first_exon_end
        match2 = read.qlen - match1

        # match lies fully in one exon - ignore
        if match1 <= 0 or match2 <= 0:
            nfull += 1
            continue

        # increment pos
        read.pos = first_exon_start + read.pos
        read.tid = contig2tid[contig]
        # 3 = BAM_CREF_SKIP
        read.cigar = [( 0, match1), (3, intron_length), (0, match2) ]

        outfile.write( read )
      
        noutput += 1

    outfile.close()
    if genomefile:
        genomefile.close()

    c = E.Counter()
    c.input = ninput
    c.output = noutput
    c.full = nfull
    c.cigar = ncigar
    c.ambiguous = nambiguous
    c.unmapped = nunmapped

    E.info( "%s" % str(c))

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
analyze_go.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python analyze_go.py --help

Type::

   python analyze_go.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse

import CGAT.Experiment as E
import CGAT.Database as Database


def WriteBackground( go_type, options, suffix ):

    statement = """SELECT DISTINCTROW
    gene_stable_id, glook_%s_id, description, olook_evidence_code
    FROM %s.%s_go_%s__go_%s__main
    WHERE glook_%s_id IS NOT NULL
    GROUP BY gene_stable_id, glook_%s_id, description
    ORDER BY gene_stable_id
    """ % (go_type,
           options.database, options.species, go_type, go_type,
           go_type, go_type)
    
    result = dbhandle.Execute(statement).fetchall()

    outfile = open( "%s%s" % (options.prefix, suffix), "w" )
    
    for r in result:
        outfile.write( " : ".join( map(str, r) ) + "\n" )
        
    outfile.close()
    

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: analyze_go.py 309 2005-12-01 15:50:26Z andreas $")

    dbhandle = Database.Database()
    
    parser.add_option("-s", "--species", dest="species", type="string",
                      help="species to use." )

    parser.add_option("-p", "--prefix", dest="prefix", type="string",
                      help="prefix to use for temporary files." )

    parser.set_defaults( species = "dmelanogaster")
    parser.set_defaults( database = "ensembl_mart_31")
    parser.set_defaults( prefix = "dm_go_")    
    
    (options, args) = E.Start( parser, add_mysql_options = True )

    dbhandle.Connect( options )

    WriteBackground( "biol_process", options, "bp" )
    WriteBackground( "cell_location", options, "lm" )
    WriteBackground( "mol_function", options, "fm" )        


        

    



"""RLE.py - a simple run length encoder
=======================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Taken from: http://rosettacode.org/wiki/Run-length_encoding#Python
"""
from itertools import groupby
import array

def encode(input_array):
    """encode array or string.

    return tuples of (count, value).

    >>> encode(array.array( "i", (10,10,10,10,20,20,20,20) ) )
    [(4, 10), (4, 20)]

    >>> encode("aaaaahhhhhhmmmmmmmuiiiiiiiaaaaaa")
    [(5, 'a'), (6, 'h'), (7, 'm'), (1, 'u'), (7, 'i'), (6, 'a')]

    """
    return [(len(list(g)), k) for k,g in groupby(input_array)]
 
def decode(lst, typecode ):
    """decode to array

    >>> decode( [(4, 10), (4, 20)], typecode="i" )
    array('i', [10, 10, 10, 10, 20, 20, 20, 20])

    >>> decode( [(5, 'a'), (6, 'h'), (7, 'm'), (1, 'u'), (7, 'i'), (6, 'a')], typecode="c" )
    array('c', 'aaaaahhhhhhmmmmmmmuiiiiiiiaaaaaa')

    """
    a = array.array( typecode )
    for n,c in lst: 
        a.extend( array.array( typecode, (c,) * n ) )
    return a

def compress( input_string, bytes = 1 ):
    """return compressed stream."""
    r = []
    for n,c in encode(input_array):
        while n > 255:
            n -= 255
            r.append( chr(255) ) 
            r.append( ord(c) )
        r.append( chr(n) ) 
        r.append( ord(c) )
    return "".join( r )

def uncompress( stream ):
    
    n,r = None, []
    for c in stream:
        if n == None:
            n = c
        else:
            r.append( n,c )
            n = None
    return decode( r )
    
if __name__ == "__main__":
    import doctest
    doctest.testmod()
 




################################################################################
#   Gene prediction pipeline 
#
#   $Id: gff2fasta.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bed2fasta.py - get sequences from bed file
==========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script outputs nucleotide sequences for intervals within
a :term:`bed` formatted file using.

Usage
-----

A required input to bed2fasta.py is a CGAT indexed genome. To obtain an
idexed human reference genome we would type

Example::
   cat hg19.fasta | index_fasta.py hg19 > hg19.log

This file would then serve as the --genome-file when we wish to extract
sequences from a :term:`bed` formatted file.


For example we could now type::

   cat in.bed | python bed2fasta.py --genome-file hg19 > out.fasta

Where we take a set of genomic intervals (e.g. from a human ChIP-seq experiment)
and output their respective nucleotide sequences.


Type::

   python bed2fasta.py --help

for command line help.

Code
----

'''
import sys
import string
import re
import optparse
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Bed as Bed
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Masker as Masker

############################################################
############################################################
############################################################
def maskSequences( sequences, masker = None):
    '''return a list of masked sequence.

    *masker* can be one of
        dust/dustmasker * run dustmasker on sequences
        softmask        * use softmask to hardmask sequences
    '''

    if masker in ("dust", "dustmasker"):
        masker_object = Masker.MaskerDustMasker()
    else:
        masker_object = None

    if masker == "softmask":
        # the genome sequence is repeat soft-masked
        masked_seq = sequences
    elif masker in ("dust", "dustmasker"):
        # run dust
        masked_seq = masker_object.maskSequences( [ x.upper() for x in sequences ] )
    elif masker == None:
        masked_seq = [x.upper() for x in sequences ]
    else:
        raise ValueError("unknown masker %s" % masker )

    # hard mask softmasked characters
    masked_seq = [re.sub( "[a-z]","N", x) for x in masked_seq ]

    return masked_seq

##------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff2fasta.py 2861 2010-02-23 17:36:32Z andreas $")

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome."  )

    parser.add_option("-m", "--masker", dest="masker", type="choice",
                      choices= ("dust", "dustmasker", "softmask", "none" ),
                      help="apply masker [%default]."  )

    parser.add_option( "-o", "--mode", dest="mode", type="choice",
                       choices = ("intervals", "leftright" ),
                       help="what to output [%default]" )

    parser.add_option( "--min-length", dest="min_length", type="int",
                       help="require a minimum sequence length [%default]" )

    parser.add_option( "--max-length", dest="max_length", type="int",
                       help="require a maximum sequence length [%default]" )

    parser.add_option( "--extend-at", dest="extend_at", type="choice",
                       choices=("none", "3", "5", "both", "3only", "5only" ),
                       help="extend at no, 3', 5' or both ends. If 3only or 5only are set, only the added sequence is returned [default=%default]" )

    parser.add_option( "--extend-by", dest="extend_by", type="int",
                       help="extend by # bases [default=%default]" )

    parser.add_option( "--use-strand", dest="ignore_strand", action="store_false",
                       help="use strand information and return reverse complement [default=%default]" )


    parser.set_defaults(
        genome_file = None,
        masker = None,
        mode = "intervals",
        min_length = 0,
        max_length = 0,
        extend_at = None,
        extend_by = 100,
        ignore_strand = True,
        )

    (options, args) = E.Start( parser )

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
        contigs = fasta.getContigSizes()
        fasta.setConverter( IndexedFasta.getConverter( "zero-both-open") )

    counter = E.Counter()
    ids, seqs = [], []

    E.info( "collecting sequences" )
    for bed in Bed.setName( Bed.iterator( options.stdin )):
        counter.input += 1

        lcontig = fasta.getLength( bed.contig )

        if options.ignore_strand:
            strand = "+"
        else:
            strand = bed.strand

        if options.mode == "intervals":
            ids.append( "%s %s:%i..%i (%s)" % (bed.name, bed.contig, bed.start, bed.end, strand) )
            seqs.append( fasta.getSequence( bed.contig, strand, bed.start, bed.end) )

        elif options.mode == "leftright":
            l = bed.end - bed.start

            start, end = max(0,bed.start-l), bed.end-l
            ids.append( "%s_l %s:%i..%i (%s)" % (bed.name, bed.contig, start, end, strand) )
            seqs.append( fasta.getSequence( bed.contig, strand, start, end) )
            
            start, end = bed.start+l, min(lcontig,bed.end+l)
            ids.append( "%s_r %s:%i..%i (%s)" % (bed.name, bed.contig, start, end, strand) )
            seqs.append( fasta.getSequence( bed.contig, strand, start, end) )
            
    E.info( "collected %i sequences" % len(seqs) )

    masked = maskSequences( seqs, options.masker )
    options.stdout.write("\n".join( [ ">%s\n%s" % (x,y) for x,y in zip(ids, masked) ] ) + "\n" )

    E.info( "masked %i sequences" % len(seqs) )

    counter.output = len(seqs )

    E.info( "%s" % counter )

    E.Stop()






################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
CorrespondenceAnalysis.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import numpy, numpy.linalg, numpy.linalg.linalg

##---------------------------------------------------------------------
def GetIndices( matrix ):
    """return order (1st eigenvector) of row and column indicies.

    This procedure fails if there are row or columns with a sum of 0.
    """

    nrows, ncols = matrix.shape

    # calculate row and column sums
    row_sums = numpy.sum( matrix, 1 )
    col_sums = numpy.sum( matrix, 0 )    

    # check for empty rows/columns
    # return the original permutation
    if 0 in row_sums or 0 in col_sums:
        return range(nrows), range(ncols)
    
    a = numpy.zeros( (nrows, nrows), numpy.float)
    for x in range( 0, nrows):
        a[x,x] = 1.0 / float(row_sums[x])

    b = numpy.zeros( (ncols, ncols), numpy.float)    
    for x in range( 0, ncols):
        b[x,x] = 1.0 / float(col_sums[x])
        
    M = numpy.dot( \
        a, numpy.dot( \
        matrix, numpy.dot( \
        b, numpy.transpose( matrix ))))

    try:
        row_eigenvector = numpy.linalg.eig(M)[1][:,1]
    except numpy.linalg.linalg.LinAlgError, msg:
        raise ValueError( msg )
 
    M = numpy.dot( \
        b, numpy.dot( \
            numpy.transpose(matrix), numpy.dot( \
                a, matrix )))

    try:
        col_eigenvector = numpy.linalg.eig(M)[1][:,1]
    except numpy.linalg.linalg.LinAlgError, msg:
        raise ValueError( msg )
         
    ## insert columns ignored at the computation and give them the lowest
    ## eigenvalue
    row_eigenvector = row_eigenvector.astype(numpy.float) 
    col_eigenvector = col_eigenvector.astype(numpy.float)

    return row_eigenvector, col_eigenvector

##---------------------------------------------------------------------
def GetPermutatedMatrix( matrix, 
                         map_row_new2old, map_col_new2old,
                         row_headers = None, col_headers = None):
    """return a permuted matrix. Note, that currently this is very
    inefficient, as I do not know how to do this in numpy.
    """

    nrows, ncols = matrix.shape

    result = numpy.zeros( (nrows, ncols), matrix.dtype)
    for r in range(0, nrows):
        for c in range(0,ncols):
            result[r,c] = matrix[map_row_new2old[r], map_col_new2old[c]]

    if not row_headers or not col_headers:
        return result

    rows = []
    for x in map_row_new2old:
        rows.append( row_headers[x] )
        
    cols = []
    for x in map_col_new2old:
        cols.append( col_headers[x] )

    return result, rows, cols
        
##---------------------------------------------------------------------
def PermuteRows( matrix ):
    pass
    

if __name__ == "__main__":

    num_rows = 6
    num_cols = 5
    matrix = numpy.zeros( (num_rows,num_cols), numpy.int)

    matrix[0,2] = 1
    matrix[0,3] = 1
    matrix[1,0] = 1
    matrix[1,1] = 1
    matrix[1,4] = 1
    matrix[2,1:5] = 1
    matrix[3,2:4] = 1
    matrix[4,0] = 1
    matrix[4,4] = 1
    matrix[5,0] = 1
    matrix[5,2:5] = 1


    print "matrix=", matrix


    row_indices, col_indices =  GetIndices( matrix )

    map_row_new2old = numpy.argsort(row_indices)
    map_col_new2old = numpy.argsort(col_indices)

    print map_row_new2old
    print map_col_new2old

    print GetPermutatedMatrix( matrix, map_row_new2old, map_col_new2old)

    

